@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2019-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2019-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -16,23 +16,157 @@
 ##  along with this program.  If not, see
 ##  <http://www.gnu.org/licenses/>.
 
-from __future__ import unicode_literals
-
 from collections import defaultdict
+import locale
 import logging
 import os
 
 import requests
 
-from photini.metadata import Location
-from photini.photinimap import LatLongDisplay
-from photini.pyqt import (
-    catch_all, Qt, QtCore, QtGui, QtSignal, QtSlot, QtWidgets, SingleLineEdit)
+from photini.configstore import key_store
+from photini.metadata import ImageMetadata
+from photini.photinimap import GeocoderBase, LatLongDisplay
+from photini.pyqt import *
+from photini.types import MD_Location
+from photini.widgets import CompactButton, LatLongDisplay, SingleLineEdit
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
 
 
+class OpenCage(GeocoderBase):
+    api_key = key_store.get('opencage', 'api_key')
+
+    def query(self, params):
+        params['key'] = self.api_key
+        params['abbrv'] = '1'
+        params['no_annotations'] = '1'
+        with Busy():
+            self.rate_limit()
+            try:
+                rsp = requests.get(
+                    'https://api.opencagedata.com/geocode/v1/json',
+                    params=params, timeout=5)
+            except Exception as ex:
+                logger.error(str(ex))
+                return []
+        if rsp.status_code >= 400:
+            logger.error('Search error %d', rsp.status_code)
+            return []
+        rsp = rsp.json()
+        status = rsp['status']
+        if status['code'] != 200:
+            logger.error(
+                'Search error %d: %s', status['code'], status['message'])
+            return []
+        if rsp['total_results'] < 1:
+            logger.error('No results found')
+            return []
+        rate = rsp['rate']
+        self.block_timer.setInterval(
+            5000 * rate['limit'] // max(rate['remaining'], 1))
+        return rsp['results']
+
+    # Map OpenCage address components to IPTC address heirarchy. There
+    # are many possible components (user generated data) so any
+    # unrecognised ones are put in 'SubLocation'. See
+    # https://github.com/OpenCageData/address-formatting/blob/master/conf/components.yaml
+    address_map = {
+        'WorldRegion'   :('continent',),
+        'CountryCode'   :('ISO_3166-1_alpha-3', 'ISO_3166-1_alpha-2',
+                          'country_code'),
+        'CountryName'   :('country', 'country_name'),
+        'ProvinceState' :('county', 'county_code', 'local_administrative_area',
+                          'state_district', 'state', 'state_code', 'province',
+                          'region', 'island'),
+        'City'          :('neighbourhood', 'city_block', 'quarter', 'suburb',
+                          'district', 'borough', 'city_district', 'commercial',
+                          'industrial', 'houses', 'subdivision',
+                          'village', 'town', 'municipality', 'city',
+                          'partial_postcode', 'postcode'),
+        'SubLocation'   :('house_number', 'street_number',
+                          'house', 'public_building', 'building', 'residential',
+                          'water', 'road', 'pedestrian', 'path',
+                          'street_name', 'street', 'cycleway', 'footway',
+                          'place', 'square',
+                          'locality', 'hamlet', 'croft'),
+        'ignore'        :('ISO_3166-2', 'political_union', 'road_reference',
+                          'road_reference_intl', 'road_type',
+                          '_category', '_type'),
+        }
+
+    def get_address(self, coords):
+        coords = [float(x) for x in coords.split(',')]
+        params = {'q': '{:.5f},{:.5f}'.format(*coords)}
+        lang, encoding = locale.getdefaultlocale()
+        if lang:
+            params['language'] = lang
+        results = self.cached_query(params)
+        if not results:
+            return None
+        address = dict(results[0]['components'])
+        formatted = results[0]['formatted']
+        for key in address.keys():
+            if isinstance(address[key], list):
+                try:
+                    address[key] = '; '.join(address[key])
+                except Exception:
+                    pass
+            if not isinstance(address[key], str):
+                try:
+                    address[key] = str(address[key])
+                except Exception:
+                    del address[key]
+        # remove some known equivalent data
+        for key_1, key_2 in (('county_code', 'county'),
+                             ('state_code', 'state'),
+                             ('partial_postcode', 'postcode')):
+            if key_1 in address and key_2 in address:
+                del address[key_1]
+        # remove duplicate data
+        for key_list in self.address_map.values():
+            for n, key_1 in enumerate(key_list):
+                if key_1 not in address:
+                    continue
+                for key_2 in key_list[n+1:]:
+                    if key_2 not in address:
+                        continue
+                    if address[key_1] == address[key_2]:
+                        del address[key_1]
+                        break
+        # attempt to format postcode correctly
+        for key in 'city', 'state', 'country':
+            if 'postcode' in address and key in address:
+                for fmt in '{0} {1}', '{0}, {1}', '{1} {0}', '{1}, {0}':
+                    guess = fmt.format(address['postcode'], address[key])
+                    if guess in formatted:
+                        address[key] = guess
+                        del address['postcode']
+                        break
+        return MD_Location.from_address(address, self.address_map)
+
+    def search_terms(self):
+        text = self.tr('Address lookup powered by OpenCage')
+        tou_opencage = CompactButton(text)
+        tou_opencage.clicked.connect(self.load_tou_opencage)
+        tou_osm = CompactButton(
+            self.tr('Geodata © OpenStreetMap contributors'))
+        tou_osm.clicked.connect(self.load_tou_osm)
+        return [tou_opencage, tou_osm]
+
+    @QtSlot()
+    @catch_all
+    def load_tou_opencage(self):
+        QtGui.QDesktopServices.openUrl(
+            QtCore.QUrl('https://geocoder.opencagedata.com/'))
+
+    @QtSlot()
+    @catch_all
+    def load_tou_osm(self):
+        QtGui.QDesktopServices.openUrl(
+            QtCore.QUrl('http://www.openstreetmap.org/copyright'))
+
+
 class LocationInfo(QtWidgets.QWidget):
     new_value = QtSignal(object, dict)
 
@@ -42,11 +176,26 @@ class LocationInfo(QtWidgets.QWidget):
         self.setLayout(layout)
         layout.setContentsMargins(0, 0, 0, 0)
         self.members = {}
-        for key in ('sublocation', 'city', 'province_state',
-                    'country_name', 'country_code', 'world_region'):
-            self.members[key] = SingleLineEdit()
-            self.members[key].editingFinished.connect(self.editing_finished)
-        self.members['country_code'].setMaximumWidth(40)
+        for key in ('SubLocation', 'City', 'ProvinceState',
+                    'CountryName', 'CountryCode', 'WorldRegion'):
+            self.members[key] = SingleLineEdit(
+                key, length_check=ImageMetadata.max_bytes(key))
+            self.members[key].new_value.connect(self.editing_finished)
+        self.members['CountryCode'].setMaximumWidth(
+            width_for_text(self.members['CountryCode'], 'W' * 4))
+        self.members['SubLocation'].setToolTip('<p>' + translate(
+            'AddressTab', 'Enter the name of the sublocation.') + '</p>')
+        self.members['City'].setToolTip('<p>' + translate(
+            'AddressTab', 'Enter the name of the city.') + '</p>')
+        self.members['ProvinceState'].setToolTip('<p>' + translate(
+            'AddressTab', 'Enter the name of the province or state.') + '</p>')
+        self.members['CountryName'].setToolTip('<p>' + translate(
+            'AddressTab', 'Enter the name of the country.') + '<p>')
+        self.members['CountryCode'].setToolTip('<p>' + translate(
+            'AddressTab', 'Enter the 2 or 3 letter ISO 3166 country code'
+            ' of the country.') + '</p>')
+        self.members['WorldRegion'].setToolTip('<p>' + translate(
+            'AddressTab', 'Enter the name of the world region.') + '</p>')
         for j, text in enumerate((
                 translate('AddressTab', 'Street'),
                 translate('AddressTab', 'City'),
@@ -57,12 +206,12 @@ class LocationInfo(QtWidgets.QWidget):
             label = QtWidgets.QLabel(text)
             label.setAlignment(Qt.AlignRight)
             layout.addWidget(label, j, 0)
-        layout.addWidget(self.members['sublocation'], 0, 1, 1, 2)
-        layout.addWidget(self.members['city'], 1, 1, 1, 2)
-        layout.addWidget(self.members['province_state'], 2, 1, 1, 2)
-        layout.addWidget(self.members['country_name'], 3, 1)
-        layout.addWidget(self.members['country_code'], 3, 2)
-        layout.addWidget(self.members['world_region'], 4, 1, 1, 2)
+        layout.addWidget(self.members['SubLocation'], 0, 1, 1, 2)
+        layout.addWidget(self.members['City'], 1, 1, 1, 2)
+        layout.addWidget(self.members['ProvinceState'], 2, 1, 1, 2)
+        layout.addWidget(self.members['CountryName'], 3, 1)
+        layout.addWidget(self.members['CountryCode'], 3, 2)
+        layout.addWidget(self.members['WorldRegion'], 4, 1, 1, 2)
         layout.setRowStretch(5, 1)
 
     def get_value(self):
@@ -73,9 +222,10 @@ class LocationInfo(QtWidgets.QWidget):
             new_value[key] = self.members[key].get_value().strip() or None
         return new_value
 
-    @QtSlot()
+    @QtSlot(str, object)
     @catch_all
-    def editing_finished(self):
+    def editing_finished(self, key, value):
+        self.members[key].set_value(value)
         self.new_value.emit(self, self.get_value())
 
 
@@ -95,7 +245,7 @@ class TabWidget(QtWidgets.QWidget):
     def __init__(self, image_list, parent=None):
         super(TabWidget, self).__init__(parent)
         self.app = QtWidgets.QApplication.instance()
-        self.geocoder = self.app.open_cage
+        self.geocoder = OpenCage(parent=self)
         self.image_list = image_list
         self.setLayout(QtWidgets.QHBoxLayout())
         ## left side
@@ -112,7 +262,7 @@ class TabWidget(QtWidgets.QWidget):
         self.auto_location.clicked.connect(self.get_address)
         left_side.addWidget(self.auto_location, 1, 0, 1, 2)
         # terms and conditions
-        terms = self.geocoder.search_terms(search=False)
+        terms = self.geocoder.search_terms()
         left_side.addWidget(terms[0], 3, 0, 1, 2)
         left_side.addWidget(terms[1], 4, 0, 1, 2)
         left_side.setColumnStretch(1, 1)
@@ -130,18 +280,9 @@ class TabWidget(QtWidgets.QWidget):
         self.location_info.setMovable(True)
         self.location_info.setEnabled(False)
         self.layout().addWidget(self.location_info, stretch=1)
-        # other init
-        self.image_list.image_list_changed.connect(self.image_list_changed)
-
-    @QtSlot()
-    @catch_all
-    def image_list_changed(self):
-        self.coords.refresh()
-        self.auto_location.setEnabled(bool(self.coords.get_value()))
-        self.display_location()
 
     def refresh(self):
-        pass
+        self.new_selection(self.image_list.get_selected_images())
 
     def do_not_close(self):
         return False
@@ -161,7 +302,7 @@ class TabWidget(QtWidgets.QWidget):
             'AddressTab', 'Duplicate location'), self.duplicate_location)
         menu.addAction(translate(
             'AddressTab', 'Delete location'), self.delete_location)
-        action = menu.exec_(event.globalPos())
+        action = execute(menu, event.globalPos())
 
     @QtSlot()
     @catch_all
@@ -169,7 +310,7 @@ class TabWidget(QtWidgets.QWidget):
         idx = self.location_info.currentIndex()
         for image in self.image_list.get_selected_images():
             # duplicate data
-            location = Location(self._get_location(image, idx) or {})
+            location = MD_Location(self._get_location(image, idx) or {})
             # shuffle data up
             location_list = list(image.metadata.location_shown or [])
             location_list.insert(idx, location)
@@ -240,9 +381,10 @@ class TabWidget(QtWidgets.QWidget):
 
     @QtSlot(object, dict)
     @catch_all
-    def new_location(self, widget, new_value):
+    def new_location(self, widget, new_value, images=[]):
+        images = images or self.image_list.get_selected_images()
         idx = self.location_info.indexOf(widget)
-        for image in self.image_list.get_selected_images():
+        for image in images:
             temp = dict(self._get_location(image, idx) or {})
             temp.update(new_value)
             self._set_location(image, idx, temp)
@@ -253,9 +395,14 @@ class TabWidget(QtWidgets.QWidget):
     def set_tab_text(self, idx):
         if idx == 0:
             text = translate('AddressTab', 'camera')
+            tip = translate('AddressTab', 'Enter the details about a location'
+                            ' where this image was created.')
         else:
             text = translate('AddressTab', 'subject {}').format(idx)
+            tip = translate('AddressTab', 'Enter the details about a location'
+                            ' which is shown in this image.')
         self.location_info.setTabText(idx, text)
+        self.location_info.setTabToolTip(idx, '<p>' + tip + '</p>')
 
     @QtSlot()
     @catch_all
@@ -306,17 +453,17 @@ class TabWidget(QtWidgets.QWidget):
                 for key in widget.members:
                     widget.members[key].set_value(None)
 
-    @QtSlot(list)
-    @catch_all
     def new_selection(self, selection):
         self.location_info.setEnabled(bool(selection))
-        self.coords.refresh()
+        self.coords.update_display(selection)
         self.auto_location.setEnabled(bool(self.coords.get_value()))
         self.display_location()
 
     @QtSlot()
     @catch_all
     def get_address(self):
+        images = self.image_list.get_selected_images()
         location = self.geocoder.get_address(self.coords.get_value())
         if location:
-            self.new_location(self.location_info.currentWidget(), location)
+            self.new_location(
+                self.location_info.currentWidget(), location, images)
