@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -26,6 +26,7 @@ import requests
 from photini.configstore import key_store
 from photini.photinimap import GeocoderBase, PhotiniMap
 from photini.pyqt import Busy, catch_all, Qt, QtCore, QtWidgets, scale_font
+from photini.widgets import Label
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
@@ -34,7 +35,7 @@ translate = QtCore.QCoreApplication.translate
 class BingGeocoder(GeocoderBase):
     interval = 50
 
-    def query(self, url, params):
+    def query(self, params, url):
         params['key'] = self.api_key
         with Busy():
             self.rate_limit()
@@ -62,12 +63,13 @@ class BingGeocoder(GeocoderBase):
         return resource_sets
 
     def get_altitude(self, coords):
+        coords = [float(x) for x in coords.split(',')]
         params = {
-            'points' : coords.replace(' ', ''),
+            'points' : '{:.5f},{:.5f}'.format(*coords),
             'heights': 'sealevel',
             }
-        resource_sets = self.query(
-            'http://dev.virtualearth.net/REST/v1/Elevation/List', params)
+        resource_sets = self.cached_query(
+            params, 'http://dev.virtualearth.net/REST/v1/Elevation/List')
         if resource_sets:
             return resource_sets[0]['resources'][0]['elevations'][0]
         return None
@@ -75,24 +77,26 @@ class BingGeocoder(GeocoderBase):
     def search(self, search_string, bounds=None):
         params = {
             'query' : search_string,
-            'maxRes': 20,
+            'maxRes': '20',
             }
         lang, encoding = locale.getdefaultlocale()
         if lang:
             params['culture'] = lang.replace('_', '-')
         if bounds:
             north, east, south, west = bounds
-            params['userMapView'] = '{!r},{!r},{!r},{!r}'.format(
+            params['userMapView'] = '{:.4f},{:.4f},{:.4f},{:.4f}'.format(
                 south, west, north, east)
-        for resource_set in self.query(
-                'http://dev.virtualearth.net/REST/v1/Locations', params):
+        for resource_set in self.cached_query(
+                params, 'http://dev.virtualearth.net/REST/v1/Locations'):
             for resource in resource_set['resources']:
                 south, west, north, east = resource['bbox']
                 yield north, east, south, west, resource['name']
 
     def search_terms(self):
-        widget = QtWidgets.QLabel(translate(
-            'MapTabBing', 'Search and altitude lookup\nprovided by Bing'))
+        widget = Label(
+            translate(
+                'MapTabBing', 'Search and altitude lookup provided by Bing'),
+            lines=2)
         widget.setAlignment(Qt.AlignRight)
         scale_font(widget, 80)
         return [widget]
