@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -17,109 +17,52 @@
 ##  along with this program.  If not, see
 ##  <http://www.gnu.org/licenses/>.
 
-from __future__ import unicode_literals
-
-from collections import defaultdict
-from datetime import datetime
+from datetime import date, datetime
 import logging
 
-import six
-
-from photini.pyqt import (
-    catch_all, ComboBox, multiple_values, MultiLineEdit, Qt, QtCore, QtGui,
-    QtSlot, QtWidgets, SingleLineEdit, Slider)
+from photini.metadata import ImageMetadata
+from photini.pyqt import *
+from photini.widgets import (
+    ComboBox, LangAltWidget, MultiLineEdit, SingleLineEdit, Slider)
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
 
 
-class LineEdit(QtWidgets.QLineEdit):
-    def __init__(self, *arg, **kw):
-        super(LineEdit, self).__init__(*arg, **kw)
-        self.multiple_values = multiple_values()
-        self._is_multiple = False
-        self.get_value = self.text
-
-    @catch_all
-    def contextMenuEvent(self, event):
-        menu = self.createStandardContextMenu()
-        suggestion_group = QtWidgets.QActionGroup(menu)
-        if self._is_multiple:
-            if self.choices:
-                sep = menu.insertSeparator(menu.actions()[0])
-                fm = menu.fontMetrics()
-                for suggestion in self.choices:
-                    label = six.text_type(suggestion).replace('\n', ' ')
-                    label = fm.elidedText(label, Qt.ElideMiddle, self.width())
-                    action = QtWidgets.QAction(label, suggestion_group)
-                    action.setData(six.text_type(suggestion))
-                    menu.insertAction(sep, action)
-        action = menu.exec_(event.globalPos())
-        if action and action.actionGroup() == suggestion_group:
-            self.set_value(action.data())
-
-    def set_value(self, value):
-        self._is_multiple = False
-        if not value:
-            self.clear()
-            self.setPlaceholderText('')
-        else:
-            self.setText(six.text_type(value))
-
-    def set_multiple(self, choices=[]):
-        self._is_multiple = True
-        self.choices = list(choices)
-        self.setPlaceholderText(self.multiple_values)
-        self.clear()
-
-    def is_multiple(self):
-        return self._is_multiple and not bool(self.get_value())
-
-
-class LineEditWithAuto(QtWidgets.QWidget):
-    def __init__(self, *arg, **kw):
-        super(LineEditWithAuto, self).__init__(*arg, **kw)
-        self._is_multiple = False
-        layout = QtWidgets.QHBoxLayout()
-        layout.setContentsMargins(0, 0, 0, 0)
-        self.setLayout(layout)
-        # line edit box
-        self.edit = LineEdit()
-        layout.addWidget(self.edit)
-        # auto complete button
-        self.auto = QtWidgets.QPushButton(translate('DescriptiveTab', 'Auto'))
-        layout.addWidget(self.auto)
-        # adopt child widget methods and signals
-        self.set_value = self.edit.set_value
-        self.get_value = self.edit.get_value
-        self.set_multiple = self.edit.set_multiple
-        self.is_multiple = self.edit.is_multiple
-        self.editingFinished = self.edit.editingFinished
-        self.autoComplete = self.auto.clicked
-
-
 class RatingWidget(QtWidgets.QWidget):
-    def __init__(self, *arg, **kw):
+    new_value = QtSignal(str, object)
+
+    def __init__(self, key, *arg, **kw):
         super(RatingWidget, self).__init__(*arg, **kw)
+        self._key = key
         self.multiple_values = multiple_values()
         self.setLayout(QtWidgets.QHBoxLayout())
         self.layout().setContentsMargins(0, 0, 0, 0)
         # slider
         self.slider = Slider(Qt.Horizontal)
-        self.slider.setFixedWidth(200)
+        self.slider.setFixedWidth(width_for_text(self.slider, 'x' * 25))
         self.slider.setRange(-2, 5)
         self.slider.setPageStep(1)
         self.slider.valueChanged.connect(self.set_display)
+        self.slider.editing_finished.connect(self._new_value)
         self.layout().addWidget(self.slider)
         # display
         self.display = QtWidgets.QLineEdit()
+        if self.display.isRightToLeft():
+            self.display.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
+        self.display.setStyleSheet("* {background-color:rgba(0,0,0,0);}")
         self.display.setFrame(False)
         self.display.setReadOnly(True)
+        self.display.setContextMenuPolicy(Qt.NoContextMenu)
         self.display.setFocusPolicy(Qt.NoFocus)
         self.layout().addWidget(self.display)
         # adopt child methods/signals
         self.is_multiple = self.slider.is_multiple
-        self.editing_finished = self.slider.editing_finished
+
+    @QtSlot()
+    @catch_all
+    def _new_value(self):
+        self.new_value.emit(self._key, self.get_value())
 
     @QtSlot(int)
     @catch_all
@@ -130,8 +73,8 @@ class RatingWidget(QtWidgets.QWidget):
         elif value == -1:
             self.display.setText(translate('DescriptiveTab', 'reject'))
         else:
-            self.display.setText((six.unichr(0x2605) * value) +
-                                 (six.unichr(0x2606) * (5 - value)))
+            self.display.setText((chr(0x2605) * value) +
+                                 (chr(0x2606) * (5 - value)))
 
     def set_value(self, value):
         if not value:
@@ -153,23 +96,29 @@ class RatingWidget(QtWidgets.QWidget):
 
 
 class KeywordsEditor(QtWidgets.QWidget):
-    def __init__(self, *arg, **kw):
-        super(KeywordsEditor, self).__init__(*arg, **kw)
+    def __init__(self, key, **kw):
+        super(KeywordsEditor, self).__init__()
         self.config_store = QtWidgets.QApplication.instance().config_store
-        self.league_table = defaultdict(int)
-        for keyword, score in eval(self.config_store.get(
-            'descriptive', 'keywords', '{}')).items():
-            self.league_table[keyword] = score
+        self.league_table = {}
+        for keyword, score in self.config_store.get(
+                                'descriptive', 'keywords', {}).items():
+            if isinstance(score, int):
+                # old style keyword list
+                self.league_table[keyword] = date.min.isoformat(), score // 50
+            else:
+                # new style keyword list
+                self.league_table[keyword] = score
         layout = QtWidgets.QHBoxLayout()
         layout.setContentsMargins(0, 0, 0, 0)
         self.setLayout(layout)
         # line edit box
-        self.edit = SingleLineEdit(spell_check=True)
+        self.edit = SingleLineEdit(key, **kw)
         layout.addWidget(self.edit)
         # favourites drop down
         self.favourites = ComboBox()
+        self.favourites.addItem(translate('DescriptiveTab', '<favourites>'))
+        self.favourites.setFixedWidth(width_for_text(self.favourites, 'x' * 16))
         self.update_favourites()
-        self.favourites.setFixedWidth(self.favourites.title_width())
         self.favourites.currentIndexChanged.connect(self.add_favourite)
         layout.addWidget(self.favourites)
         # adopt child widget methods and signals
@@ -177,7 +126,7 @@ class KeywordsEditor(QtWidgets.QWidget):
         self.set_value = self.edit.set_value
         self.set_multiple = self.edit.set_multiple
         self.is_multiple = self.edit.is_multiple
-        self.editingFinished = self.edit.editingFinished
+        self.new_value = self.edit.new_value
 
     def update_favourites(self):
         self.favourites.clear()
@@ -198,19 +147,19 @@ class KeywordsEditor(QtWidgets.QWidget):
         self.favourites.set_dropdown_width()
 
     def update_league_table(self, images):
+        today = date.today().isoformat()
         for image in images:
-            value = list(filter(lambda x: not x.startswith('flickr:photo_id'),
-                                image.metadata.keywords or []))
+            keywords = image.metadata.keywords or []
+            value = [x for x in keywords if ':' not in x]
             if not value:
                 continue
-            for keyword in self.league_table:
-                self.league_table[keyword] = max(
-                    self.league_table[keyword] - 1, -5)
             for keyword in value:
-                self.league_table[keyword] = min(
-                    self.league_table[keyword] + 10, 1000)
-        self.config_store.set(
-            'descriptive', 'keywords', six.text_type(dict(self.league_table)))
+                if keyword not in self.league_table:
+                    self.league_table[keyword] = today, 1
+                elif self.league_table[keyword][0] != today:
+                    self.league_table[keyword] = (
+                        today, self.league_table[keyword][1] + 1)
+        self.config_store.set('descriptive', 'keywords', self.league_table)
         self.update_favourites()
 
     @QtSlot(int)
@@ -224,7 +173,7 @@ class KeywordsEditor(QtWidgets.QWidget):
         if current_value:
             new_value = current_value + '; ' + new_value
         self.set_value(new_value)
-        self.editingFinished.emit()
+        self.new_value.emit(self.edit._key, new_value)
 
 
 class TabWidget(QtWidgets.QWidget):
@@ -234,50 +183,69 @@ class TabWidget(QtWidgets.QWidget):
 
     def __init__(self, image_list, *arg, **kw):
         super(TabWidget, self).__init__(*arg, **kw)
-        self.config_store = QtWidgets.QApplication.instance().config_store
         self.image_list = image_list
-        self.form = QtWidgets.QFormLayout()
-        self.setLayout(self.form)
+        self.form = FormLayout()
+        self.setLayout(QtWidgets.QVBoxLayout())
+        self.layout().addLayout(self.form)
+        self.layout().addStretch(1)
         # construct widgets
         self.widgets = {}
         # title
-        self.widgets['title'] = SingleLineEdit(spell_check=True)
-        self.widgets['title'].editingFinished.connect(self.new_title)
+        self.widgets['title'] = LangAltWidget(
+            'title', multi_line=False, spell_check=True,
+            length_check=ImageMetadata.max_bytes('title'))
+        self.widgets['title'].setToolTip('<p>' + translate(
+            'DescriptiveTab', 'Enter a short verbal and human readable name'
+            ' for the image, this may be the file name.') + '</p>')
+        self.widgets['title'].new_value.connect(self.new_value)
         self.form.addRow(translate(
             'DescriptiveTab', 'Title / Object Name'), self.widgets['title'])
-        # description
-        self.widgets['description'] = MultiLineEdit(spell_check=True)
-        self.widgets['description'].editingFinished.connect(self.new_description)
+        # headline
+        self.widgets['headline'] = MultiLineEdit(
+            'headline', spell_check=True,
+            length_check=ImageMetadata.max_bytes('headline'))
+        self.widgets['headline'].set_height(3)
+        self.widgets['headline'].setToolTip('<p>' + translate(
+            'DescriptiveTab', 'Enter a brief publishable synopsis or summary'
+            ' of the contents of the image.') + '</p>')
+        self.widgets['headline'].new_value.connect(self.new_value)
         self.form.addRow(translate(
-            'DescriptiveTab', 'Description / Caption'), self.widgets['description'])
+            'DescriptiveTab', 'Headline'), self.widgets['headline'])
+        # description
+        self.widgets['description'] = LangAltWidget(
+            'description', spell_check=True,
+            length_check=ImageMetadata.max_bytes('description'))
+        self.widgets['description'].setToolTip('<p>' + translate(
+            'DescriptiveTab', 'Enter a "caption" describing the who, what,'
+            ' and why of what is happening in this image, this might include'
+            ' names of people, and/or their role in the action that is taking'
+            ' place within the image.') + '</p>')
+        self.widgets['description'].new_value.connect(self.new_value)
+        self.form.addRow(
+            translate('DescriptiveTab', 'Description / Caption'),
+            self.widgets['description'])
         # keywords
-        self.widgets['keywords'] = KeywordsEditor()
-        self.widgets['keywords'].editingFinished.connect(self.new_keywords)
+        self.widgets['keywords'] = KeywordsEditor(
+            'keywords', spell_check=True, multi_string=True,
+            length_check=ImageMetadata.max_bytes('keywords'))
+        self.widgets['keywords'].setToolTip('<p>' + translate(
+            'DescriptiveTab', 'Enter any number of keywords, terms or phrases'
+            ' used to express the subject matter in the image.'
+            ' Separate them with ";" characters.') + '</p>')
+        self.widgets['keywords'].new_value.connect(self.new_value)
         self.form.addRow(translate(
             'DescriptiveTab', 'Keywords'), self.widgets['keywords'])
         self.image_list.image_list_changed.connect(self.image_list_changed)
         # rating
-        self.widgets['rating'] = RatingWidget()
-        self.widgets['rating'].editing_finished.connect(self.new_rating)
+        self.widgets['rating'] = RatingWidget('rating')
+        self.widgets['rating'].new_value.connect(self.new_value)
         self.form.addRow(translate(
             'DescriptiveTab', 'Rating'), self.widgets['rating'])
-        # copyright
-        self.widgets['copyright'] = LineEditWithAuto()
-        self.widgets['copyright'].editingFinished.connect(self.new_copyright)
-        self.widgets['copyright'].autoComplete.connect(self.auto_copyright)
-        self.form.addRow(translate(
-            'DescriptiveTab', 'Copyright'), self.widgets['copyright'])
-        # creator
-        self.widgets['creator'] = LineEditWithAuto()
-        self.widgets['creator'].editingFinished.connect(self.new_creator)
-        self.widgets['creator'].autoComplete.connect(self.auto_creator)
-        self.form.addRow(translate(
-            'DescriptiveTab', 'Creator / Artist'), self.widgets['creator'])
         # disable until an image is selected
         self.setEnabled(False)
 
     def refresh(self):
-        pass
+        self.new_selection(self.image_list.get_selected_images())
 
     def do_not_close(self):
         return False
@@ -288,92 +256,17 @@ class TabWidget(QtWidgets.QWidget):
         self.widgets['keywords'].update_league_table(
             self.image_list.get_images())
 
-    @QtSlot()
+    @QtSlot(str, object)
     @catch_all
-    def new_title(self):
-        self._new_value('title')
-
-    @QtSlot()
-    @catch_all
-    def new_description(self):
-        self._new_value('description')
-
-    @QtSlot()
-    @catch_all
-    def new_keywords(self):
-        self._new_value('keywords')
-        self.widgets['keywords'].update_league_table(
-            self.image_list.get_selected_images())
-
-    @QtSlot()
-    @catch_all
-    def new_rating(self):
-        self._new_value('rating')
-
-    @QtSlot()
-    @catch_all
-    def new_copyright(self):
-        self._new_value('copyright')
-
-    @QtSlot()
-    @catch_all
-    def new_creator(self):
-        self._new_value('creator')
-
-    @QtSlot()
-    @catch_all
-    def auto_copyright(self):
-        name = self.config_store.get('user', 'copyright_name')
-        if not name:
-            name, OK = QtWidgets.QInputDialog.getText(
-                self, translate('DescriptiveTab', 'Photini: input name'),
-                translate('DescriptiveTab',
-                          "Please type in the copyright holder's name"),
-                text=self.config_store.get('user', 'creator_name', ''))
-            if OK and name:
-                self.config_store.set('user', 'copyright_name', name)
-            else:
-                name = ''
-        copyright_text = self.config_store.get(
-            'user', 'copyright_text',
-            translate('DescriptiveTab',
-                      'Copyright ©{year} {name}. All rights reserved.'))
-        for image in self.image_list.get_selected_images():
-            date_taken = image.metadata.date_taken
-            if date_taken is None:
-                date_taken = datetime.now()
-            else:
-                date_taken = date_taken['datetime']
-            value = copyright_text.format(year=date_taken.year, name=name)
-            image.metadata.copyright = value
-        self._update_widget('copyright')
-
-    @QtSlot()
-    @catch_all
-    def auto_creator(self):
-        name = self.config_store.get('user', 'creator_name')
-        if not name:
-            name, OK = QtWidgets.QInputDialog.getText(
-                self, translate('DescriptiveTab', 'Photini: input name'),
-                translate('DescriptiveTab', "Please type in the creator's name"),
-                text=self.config_store.get('user', 'copyright_name', ''))
-            if OK and name:
-                self.config_store.set('user', 'creator_name', name)
-            else:
-                name = ''
-        for image in self.image_list.get_selected_images():
-            image.metadata.creator = name
-        self._update_widget('creator')
-
-    def _new_value(self, key):
-        if not self.widgets[key].is_multiple():
-            value = self.widgets[key].get_value()
-            for image in self.image_list.get_selected_images():
-                setattr(image.metadata, key, value)
-        self._update_widget(key)
-
-    def _update_widget(self, key):
+    def new_value(self, key, value):
         images = self.image_list.get_selected_images()
+        for image in images:
+            setattr(image.metadata, key, value)
+        self._update_widget(key, images)
+        if key == 'keywords':
+            self.widgets[key].update_league_table(images)
+
+    def _update_widget(self, key, images):
         if not images:
             return
         values = []
@@ -382,12 +275,10 @@ class TabWidget(QtWidgets.QWidget):
             if value not in values:
                 values.append(value)
         if len(values) > 1:
-            self.widgets[key].set_multiple(choices=filter(None, values))
+            self.widgets[key].set_multiple(choices=[x for x in values if x])
         else:
             self.widgets[key].set_value(values[0])
 
-    @QtSlot(list)
-    @catch_all
     def new_selection(self, selection):
         if not selection:
             for key in self.widgets:
@@ -395,5 +286,5 @@ class TabWidget(QtWidgets.QWidget):
             self.setEnabled(False)
             return
         for key in self.widgets:
-            self._update_widget(key)
+            self._update_widget(key, selection)
         self.setEnabled(True)
