@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -18,15 +18,12 @@
 
 from __future__ import unicode_literals
 
-import six
 import importlib
 import logging
 from optparse import OptionParser
 import os
-import pprint
+import socket
 import sys
-from six.moves.urllib.request import getproxies
-from six.moves.urllib.parse import urlparse
 import warnings
 
 import pkg_resources
@@ -35,13 +32,11 @@ from photini import __version__, build
 from photini.configstore import BaseConfigStore
 from photini.editsettings import EditSettings
 from photini.ffmpeg import ffmpeg_version
-from photini.gi import gi_version
 from photini.imagelist import ImageList
 from photini.loggerwindow import LoggerWindow
-from photini.opencage import OpenCage
-from photini.pyqt import (
-    catch_all, Qt, QtCore, QtGui, QNetworkProxy, QtSlot, QtWidgets, qt_version,
-    qt_version_info, width_for_text)
+from photini.metadata import exiv2_version, ImageMetadata
+from photini.pyqt import *
+from photini.pyqt import QtNetwork, qt_version, qt_version_info
 from photini.spelling import SpellCheck, spelling_version
 
 try:
@@ -54,15 +49,11 @@ except ImportError as ex:
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
 
-if six.PY2:
-    logger.warning('Photini is currently using Python 2.'
-                   ' Support for Python 2 will be withdrawn in a future release.')
-
 
 class QTabBar(QtWidgets.QTabBar):
     def tabSizeHint(self, index):
         size = super(QTabBar, self).tabSizeHint(index)
-        size.setWidth(max(size.width(), width_for_text(self, 'x' * 13)))
+        size.setWidth(max(size.width(), width_for_text(self, 'x' * 10)))
         return size
 
 
@@ -90,6 +81,73 @@ class ConfigStore(BaseConfigStore, QtCore.QObject):
         super(ConfigStore, self).save()
 
 
+class ServerSocket(QtCore.QObject):
+    new_files = QtSignal(list)
+
+    def __init__(self, socket, *arg, **kw):
+        super(ServerSocket, self).__init__(*arg, **kw)
+        self.socket = socket
+        self.data = b''
+        self.socket.setParent(self)
+        self.socket.readyRead.connect(self.read_data)
+        self.socket.disconnected.connect(self.deleteLater)
+
+    @QtSlot()
+    @catch_all
+    def read_data(self):
+        file_list = []
+        while self.socket.bytesAvailable():
+            self.data += self.socket.readAll().data()
+            while b'\n' in self.data:
+                line, sep, self.data = self.data.partition(b'\n')
+                string = line.decode('utf-8')
+                file_list.append(string)
+        if file_list:
+            self.new_files.emit(file_list)
+
+
+class InstanceServer(QtNetwork.QTcpServer):
+    new_files = QtSignal(list)
+
+    def __init__(self, *arg, **kw):
+        super(InstanceServer, self).__init__(*arg, **kw)
+        config = BaseConfigStore('instance')
+        self.newConnection.connect(self.new_connection)
+        if not self.listen(QtNetwork.QHostAddress.LocalHost):
+            logger.error('Failed to start instance server:', self.errorString())
+            return
+        config.set('server', 'port', self.serverPort())
+        config.save()
+
+    @QtSlot()
+    @catch_all
+    def new_connection(self):
+        window = self.parent().window()
+        window.setWindowState(
+            (window.windowState() & ~Qt.WindowMinimized) | Qt.WindowActive)
+        window.raise_()
+        while self.hasPendingConnections():
+            socket = self.nextPendingConnection()
+            socket = ServerSocket(socket, parent=self)
+            socket.new_files.connect(self.new_files)
+
+
+def SendToInstance(files):
+    config = BaseConfigStore('instance')
+    port = config.get('server', 'port')
+    if not port:
+        return False
+    try:
+        sock = socket.create_connection(('127.0.0.1', int(port)), timeout=1000)
+    except ConnectionRefusedError:
+        return False
+    for path in files:
+        data = os.path.abspath(path).encode('utf-8') + b'\n'
+        sock.sendall(data)
+    sock.close()
+    return True
+
+
 class MainWindow(QtWidgets.QMainWindow):
     def __init__(self, options, initial_files):
         super(MainWindow, self).__init__()
@@ -104,26 +162,34 @@ class MainWindow(QtWidgets.QMainWindow):
         # logger window
         self.loggerwindow = LoggerWindow(options.verbose)
         self.loggerwindow.setWindowIcon(icon)
-        # set network proxy
-        proxies = getproxies()
-        if 'http' in proxies:
-            parsed = urlparse(proxies['http'])
-            QNetworkProxy.setApplicationProxy(QNetworkProxy(
-                QNetworkProxy.HttpProxy, parsed.hostname, parsed.port))
         # create shared global objects
         self.app = QtWidgets.QApplication.instance()
         self.app.config_store = ConfigStore('editor', parent=self)
         self.app.spell_check = SpellCheck(parent=self)
-        self.app.open_cage = OpenCage(parent=self)
+        if GpxImporter:
+            self.app.gpx_importer = GpxImporter(parent=self)
+        else:
+            self.app.gpx_importer = None
         self.app.options = options
-        # restore size
+        # initialise metadata handler
+        ImageMetadata.initialise(self.app.config_store, options.verbose)
+        # restore size and state
         size = self.width(), self.height()
-        self.resize(*eval(
-            self.app.config_store.get('main_window', 'size', str(size))))
+        self.resize(*self.app.config_store.get('main_window', 'size', size))
+        window_state = self.app.config_store.get('main_window', 'state', 0)
+        full_screen = window_state & int(
+            Qt.WindowMaximized | Qt.WindowFullScreen)
+        if full_screen:
+            self.setWindowState(self.windowState() | full_screen)
         # image selector
         self.image_list = ImageList()
         self.image_list.selection_changed.connect(self.new_selection)
+        self.image_list.image_list_changed.connect(self.new_image_list)
         self.image_list.new_metadata.connect(self.new_metadata)
+        # start instance server
+        instance_server = InstanceServer(parent=self)
+        instance_server.new_files.connect(
+            self.image_list.open_file_list, Qt.QueuedConnection)
         # update config file
         if self.app.config_store.config.has_section('tabs'):
             conv = {
@@ -132,7 +198,6 @@ class MainWindow(QtWidgets.QMainWindow):
                 'map_google'          : 'photini.googlemap',
                 'map_bing'            : 'photini.bingmap',
                 'map_mapbox'          : 'photini.mapboxmap',
-                'map_osm'             : 'photini.openstreetmap',
                 'address'             : 'photini.address',
                 'flickr_upload'       : 'photini.flickr',
                 'import_photos'       : 'photini.importer',
@@ -142,32 +207,34 @@ class MainWindow(QtWidgets.QMainWindow):
                     self.app.config_store.set(
                         'tabs', conv[key],
                         self.app.config_store.get('tabs', key))
-                    self.app.config_store.config.remove_option('tabs', key)
+                    self.app.config_store.delete('tabs', key)
         # prepare list of tabs and associated stuff
-        self.tab_list = []
-        default_modules = ['photini.descriptive',  'photini.technical',
+        self.tab_info = {}
+        default_modules = ['photini.descriptive',  'photini.ownership',
+                           'photini.technical',
                            'photini.googlemap',    'photini.bingmap',
-                           'photini.mapboxmap',    'photini.openstreetmap',
-                           'photini.address',      'photini.flickr',
+                           'photini.mapboxmap',    'photini.address',
+                           'photini.flickr',       'photini.ipernity',
                            'photini.googlephotos', 'photini.importer']
-        modules = eval(self.app.config_store.get(
-            'tabs', 'modules', pprint.pformat(default_modules)))
-        for n, module in enumerate(default_modules):
-            if module not in modules:
-                modules = list(modules)
-                modules.insert(n, module)
-                self.app.config_store.set(
-                    'tabs', 'modules', pprint.pformat(modules))
-        for module in modules:
-            tab = {'module': module}
+        self.modules = self.app.config_store.get(
+            'tabs', 'modules', default_modules)
+        if 'photini.openstreetmap' in self.modules:
+            self.modules.remove('photini.openstreetmap')
+            self.app.config_store.delete('tabs', 'photini.openstreetmap')
+        self.modules += [x for x in default_modules if x not in self.modules]
+        self.app.config_store.set('tabs', 'modules', self.modules)
+        for module in self.modules:
+            tab = {}
             try:
-                mod = importlib.import_module(tab['module'])
+                mod = importlib.import_module(module)
                 tab['class'] = mod.TabWidget
-                tab['name'] = tab['class'].tab_name()
+                tab['label'] = tab['class'].tab_name()
+                tab['name'] = tab['label'].replace('&', '')
             except ImportError as ex:
                 print(str(ex))
                 tab['class'] = None
-            self.tab_list.append(tab)
+                tab['name'] = module
+            self.tab_info[module] = tab
         # file menu
         file_menu = self.menuBar().addMenu(translate('MenuBar', 'File'))
         action = file_menu.addAction(translate('MenuBar', 'Open files'))
@@ -178,17 +245,19 @@ class MainWindow(QtWidgets.QMainWindow):
         self.save_action.setShortcuts(QtGui.QKeySequence.Save)
         self.save_action.setEnabled(False)
         self.save_action.triggered.connect(self.image_list.save_files)
+        self.fix_thumbs_action = file_menu.addAction(
+            translate('MenuBar', 'Fix missing thumbnails'))
+        self.fix_thumbs_action.setEnabled(False)
+        self.fix_thumbs_action.triggered.connect(
+            self.image_list.fix_missing_thumbs)
         action = file_menu.addAction(translate('MenuBar', 'Close all files'))
         action.triggered.connect(self.image_list.close_all_files)
-        sep = file_menu.addAction(translate('MenuBar', 'Selected images'))
-        sep.setSeparator(True)
+        file_menu.addSeparator()
+        sep = QtWidgets.QWidgetAction(self)
+        sep.setDefaultWidget(
+            QtWidgets.QLabel(translate('MenuBar', 'Selected images')))
+        file_menu.addAction(sep)
         self.selected_actions = self.image_list.add_selected_actions(file_menu)
-        if GpxImporter:
-            self.import_gpx_action = file_menu.addAction(
-                translate('MenuBar', 'Import GPX file'))
-            self.import_gpx_action.triggered.connect(self.import_pgx_file)
-        else:
-            self.import_gpx_action = None
         file_menu.addSeparator()
         action = file_menu.addAction(translate('MenuBar', 'Quit'))
         action.setShortcuts(
@@ -200,16 +269,13 @@ class MainWindow(QtWidgets.QMainWindow):
         action = options_menu.addAction(translate('MenuBar', 'Settings'))
         action.triggered.connect(self.edit_settings)
         options_menu.addSeparator()
-        for tab in self.tab_list:
-            if tab['class']:
-                name = tab['name'].replace('&', '')
-            else:
-                name = tab['module']
-            tab['action'] = options_menu.addAction(name)
+        for module in self.modules:
+            tab = self.tab_info[module]
+            tab['action'] = options_menu.addAction(tab['name'])
             tab['action'].setCheckable(True)
             if tab['class']:
-                tab['action'].setChecked(eval(
-                    self.app.config_store.get('tabs', tab['module'], 'True')))
+                tab['action'].setChecked(
+                    self.app.config_store.get('tabs', module, True))
             else:
                 tab['action'].setEnabled(False)
             tab['action'].triggered.connect(self.add_tabs)
@@ -222,29 +288,28 @@ class MainWindow(QtWidgets.QMainWindow):
         action.setCheckable(True)
         action.setChecked(self.app.spell_check.enabled)
         action.toggled.connect(self.app.spell_check.enable)
-        language_menu = spelling_menu.addMenu(
-            translate('MenuBar', 'Choose language'))
-        language_menu.setEnabled(languages is not None)
         current_language = self.app.spell_check.current_language()
         if languages:
-            language_group = QtWidgets.QActionGroup(self)
-            for name, code in languages:
-                if name != code:
-                    name = code + ': ' + name
-                action = language_menu.addAction(name)
-                action.setCheckable(True)
-                action.setChecked(code == current_language)
-                action.setData(code)
-                action.setActionGroup(language_group)
+            language_group = QtGui2.QActionGroup(self)
+            for language in sorted(languages):
+                language_menu = spelling_menu.addMenu(language)
+                for country, code in languages[language]:
+                    if country:
+                        name = '{}: {}'.format(language, country)
+                    else:
+                        name = language
+                    action = language_menu.addAction(name)
+                    action.setCheckable(True)
+                    action.setChecked(code == current_language)
+                    action.setData(code)
+                    action.setActionGroup(language_group)
             language_group.triggered.connect(self.set_language)
-        else:
-            action = language_menu.addAction(
-                translate('MenuBar', 'No dictionary installed'))
-            action.setEnabled(False)
         # help menu
         help_menu = self.menuBar().addMenu(translate('MenuBar', 'Help'))
         action = help_menu.addAction(translate('MenuBar', 'About Photini'))
         action.triggered.connect(self.about)
+        action = help_menu.addAction(translate('MenuBar', 'Check for update'))
+        action.triggered.connect(self.check_update)
         help_menu.addSeparator()
         action = help_menu.addAction(
             translate('MenuBar', 'Photini documentation'))
@@ -257,12 +322,14 @@ class MainWindow(QtWidgets.QMainWindow):
         self.tabs.setTabBar(QTabBar())
         self.tabs.setElideMode(Qt.ElideRight)
         self.tabs.currentChanged.connect(self.new_tab)
-        self.add_tabs(False)
+        self.tabs.setMovable(True)
+        self.tabs.tabBar().tabMoved.connect(self.tab_moved)
+        self.add_tabs()
         self.central_widget.addWidget(self.tabs)
         self.central_widget.addWidget(self.image_list)
         size = self.central_widget.sizes()
-        self.central_widget.setSizes(eval(
-            self.app.config_store.get('main_window', 'split', str(size))))
+        self.central_widget.setSizes(
+            self.app.config_store.get('main_window', 'split', size))
         self.central_widget.splitterMoved.connect(self.new_split)
         self.setCentralWidget(self.central_widget)
         # open files given on command line, after GUI is displayed
@@ -275,26 +342,26 @@ class MainWindow(QtWidgets.QMainWindow):
     def open_initial_files(self):
         self.image_list.open_file_list(self.initial_files)
 
-    @QtSlot(bool)
+    @QtSlot()
     @catch_all
-    def add_tabs(self, checked):
+    def add_tabs(self):
         was_blocked = self.tabs.blockSignals(True)
         current = self.tabs.currentWidget()
         self.tabs.clear()
-        idx = 0
-        for tab in self.tab_list:
+        for module in self.modules:
+            tab = self.tab_info[module]
             if not tab['class']:
-                self.app.config_store.set('tabs', tab['module'], 'True')
+                self.app.config_store.set('tabs', module, True)
                 continue
             use_tab = tab['action'].isChecked()
-            self.app.config_store.set('tabs', tab['module'], str(use_tab))
+            self.app.config_store.set('tabs', module, use_tab)
             if not use_tab:
                 continue
             if 'object' not in tab:
                 tab['object'] = tab['class'](self.image_list)
-            self.tabs.addTab(tab['object'], tab['name'])
-            self.tabs.setTabToolTip(idx, tab['name'].replace('&', ''))
-            idx += 1
+            idx = self.tabs.addTab(tab['object'], tab['label'])
+            self.tabs.setTabToolTip(idx, tab['name'])
+            self.tabs.tabBar().setTabData(idx, module)
         self.tabs.blockSignals(was_blocked)
         if current:
             self.tabs.setCurrentWidget(current)
@@ -306,12 +373,6 @@ class MainWindow(QtWidgets.QMainWindow):
         QtGui.QDesktopServices.openUrl(QtCore.QUrl(
             'http://photini.readthedocs.io/'))
 
-    @QtSlot()
-    @catch_all
-    def import_pgx_file(self):
-        importer = GpxImporter()
-        importer.do_import(self)
-
     @catch_all
     def closeEvent(self, event):
         for n in range(self.tabs.count()):
@@ -325,10 +386,10 @@ class MainWindow(QtWidgets.QMainWindow):
     @catch_all
     def edit_settings(self):
         dialog = EditSettings(self)
-        dialog.exec_()
+        execute(dialog)
         self.tabs.currentWidget().refresh()
 
-    @QtSlot(QtWidgets.QAction)
+    @QtSlot(QtGui2.QAction)
     @catch_all
     def set_language(self, action):
         self.app.spell_check.set_language(action.data())
@@ -370,13 +431,35 @@ jim@jim-easterbrook.me.uk</a><br /><br />
             'This program is released with a GNU General Public'
             ' License. For details click the "{}" button.').format(
                 dialog.buttons()[0].text()))
-        dialog.exec_()
+        execute(dialog)
+
+    @QtSlot()
+    @catch_all
+    def check_update(self):
+        import requests
+        with Busy():
+            try:
+                rsp = requests.get(
+                    'https://pypi.org/pypi/photini/json', timeout=20)
+            except:
+                logger.error(str(ex))
+                return
+        if rsp.status_code != 200:
+            logger.error('HTTP error %d', rsp.status_code)
+            return
+        release = rsp.json()['info']['version']
+        dialog = QtWidgets.QMessageBox(self)
+        dialog.setWindowTitle(translate('MenuBar', 'Photini: version check'))
+        dialog.setText(translate(
+            'MenuBar', 'You are currently running Photini version {0}. The'
+            ' latest release is {1}.').format(__version__, release))
+        execute(dialog)
 
     @QtSlot(int, int)
     @catch_all
     def new_split(self, pos, index):
         self.app.config_store.set(
-            'main_window', 'split', str(self.central_widget.sizes()))
+            'main_window', 'split', self.central_widget.sizes())
 
     @QtSlot(int)
     @catch_all
@@ -385,16 +468,34 @@ jim@jim-easterbrook.me.uk</a><br /><br />
         if current:
             self.image_list.set_drag_to_map(None)
             current.refresh()
-            self.image_list.emit_selection()
+
+    @QtSlot(int, int)
+    @catch_all
+    def tab_moved(self, new_pos, old_pos):
+        tab_bar = self.tabs.tabBar()
+        modules = []
+        for n in range(tab_bar.count()):
+            modules.append(tab_bar.tabData(n))
+        modules += [x for x in self.modules if x not in modules]
+        self.modules = modules
+        self.app.config_store.set('tabs', 'modules', self.modules)
 
     @QtSlot(list)
     @catch_all
     def new_selection(self, selection):
         self.image_list.configure_selected_actions(self.selected_actions)
-        if self.import_gpx_action:
-            self.import_gpx_action.setEnabled(len(selection) > 0)
         self.tabs.currentWidget().new_selection(selection)
 
+    @QtSlot()
+    @catch_all
+    def new_image_list(self):
+        for image in self.image_list.images:
+            thumb = image.metadata.thumbnail
+            if not thumb or not thumb['image']:
+                self.fix_thumbs_action.setEnabled(True)
+                return
+        self.fix_thumbs_action.setEnabled(False)
+
     @QtSlot(bool)
     @catch_all
     def new_metadata(self, unsaved_data):
@@ -403,8 +504,13 @@ jim@jim-easterbrook.me.uk</a><br /><br />
 
     @catch_all
     def resizeEvent(self, event):
+        window_state = int(self.windowState())
+        self.app.config_store.set('main_window', 'state', window_state)
+        if window_state & int(
+                Qt.WindowMinimized | Qt.WindowMaximized | Qt.WindowFullScreen):
+            return
         size = self.width(), self.height()
-        self.app.config_store.set('main_window', 'size', str(size))
+        self.app.config_store.set('main_window', 'size', size)
 
 
 app = None
@@ -413,13 +519,14 @@ def main(argv=None):
     global app
     if argv:
         sys.argv = argv
+    if qt_version_info < (6, 0):
+        QtWidgets.QApplication.setAttribute(Qt.AA_EnableHighDpiScaling, True)
+    QtWidgets.QApplication.setAttribute(Qt.AA_UseHighDpiPixmaps, True)
     # let Qt handle its options
     app = QtWidgets.QApplication(sys.argv)
     # get remaining argument list after Qt has swallowed its options
     sys.argv = app.arguments()
     # install translations
-    if qt_version_info < (5, 0):
-        QtCore.QTextCodec.setCodecForTr(QtCore.QTextCodec.codecForName('utf-8'))
     # English translation as a fallback (to get correct plurals)
     lang_dir = pkg_resources.resource_filename('photini', 'data/lang')
     translator = QtCore.QTranslator(parent=app)
@@ -431,55 +538,41 @@ def main(argv=None):
     if translator.load(locale, 'photini', '.', lang_dir):
         app.installTranslator(translator)
         translator = QtCore.QTranslator(parent=app)
-    # Qt's own translation, e.g. for 'apply' or 'cancel' buttons
-    if qt_version_info < (5, 0):
-        if translator.load(locale, 'qt', '_', QtCore.QLibraryInfo.location(
-                QtCore.QLibraryInfo.TranslationsPath)):
-            app.installTranslator(translator)
     # parse remaining arguments
     version = 'Photini ' + __version__ + ', build ' + build
     version += '\n  Python ' + sys.version
-    version += '\n  ' + gi_version
+    version += '\n  ' + exiv2_version
     version += '\n  ' + qt_version
     if spelling_version:
         version += '\n  ' + spelling_version
     if ffmpeg_version:
         version += '\n  ' + ffmpeg_version
-    try:
-        from photini.flickr import flickr_version
-        version += '\n  ' + flickr_version
-    except ImportError:
-        pass
     version += '\n  available styles: {}'.format(
         ', '.join(QtWidgets.QStyleFactory.keys()))
     version += '\n  using style: {}'.format(
         QtWidgets.QApplication.style().objectName())
     parser = OptionParser(
-        usage=six.text_type(translate(
-            'CLIHelp', 'Usage: %prog [options] [file_name, ...]')),
+        usage=translate('CLIHelp', 'Usage: %prog [options] [file_name, ...]'),
         version=version,
-        description=six.text_type(translate(
-            'CLIHelp', 'Photini photo metadata editor')))
+        description=translate('CLIHelp', 'Photini photo metadata editor'))
     parser.add_option(
         '-t', '--test', action='store_true',
-        help=six.text_type(translate(
-            'CLIHelp', 'test new features or API versions')))
-    parser.add_option(
-        '-u', '--utf_safe', action='store_true',
-        help=six.text_type(translate(
-            'CLIHelp', 'metadata is known to be ASCII or utf-8 encoded')))
+        help=translate('CLIHelp', 'test new features or API versions'))
     parser.add_option(
         '-v', '--verbose', action='count', default=0,
-        help=six.text_type(translate(
-            'CLIHelp', 'increase number of logging messages')))
+        help=translate('CLIHelp', 'increase number of logging messages'))
     options, args = parser.parse_args()
+    # if an instance of Photini is already running, send it the list of
+    # files to open
+    if SendToInstance(args):
+        return 0
     # ensure warnings are visible in test mode
     if options.test:
         warnings.simplefilter('default')
     # create GUI and run application event loop
     main = MainWindow(options, args)
     main.show()
-    return app.exec_()
+    return execute(app)
 
 if __name__ == "__main__":
     sys.exit(main())
