@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
 ##  along with this program.  If not, see
 ##  <http://www.gnu.org/licenses/>.
 
-from __future__ import unicode_literals
-
 import locale
 import logging
 
@@ -26,6 +24,7 @@ import requests
 from photini.configstore import key_store
 from photini.photinimap import GeocoderBase, PhotiniMap
 from photini.pyqt import Busy, Qt, QtCore, QtWidgets, scale_font
+from photini.widgets import Label
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
@@ -35,7 +34,7 @@ class GoogleGeocoder(GeocoderBase):
     api_key = key_store.get('googlemap', 'api_key')
     interval = 50
 
-    def query(self, url, params):
+    def query(self, params, url):
         params['key'] = self.api_key
         with Busy():
             self.rate_limit()
@@ -59,9 +58,10 @@ class GoogleGeocoder(GeocoderBase):
         return results
 
     def get_altitude(self, coords):
-        params = {'locations': coords.replace(' ', '')}
-        results = self.query(
-            'https://maps.googleapis.com/maps/api/elevation/json', params)
+        coords = [float(x) for x in coords.split(',')]
+        params = {'locations': '{:.5f},{:.5f}'.format(*coords)}
+        results = self.cached_query(
+            params, 'https://maps.googleapis.com/maps/api/elevation/json')
         if results:
             return results[0]['elevation']
         return None
@@ -73,19 +73,19 @@ class GoogleGeocoder(GeocoderBase):
             params['language'] = lang
         if bounds:
             north, east, south, west = bounds
-            params['bounds'] = '{!r},{!r}|{!r},{!r}'.format(
+            params['bounds'] = '{:.4f},{:.4f}|{:.4f},{:.4f}'.format(
                 south, west, north, east)
-        for result in self.query(
-                'https://maps.googleapis.com/maps/api/geocode/json', params):
+        for result in self.cached_query(
+                params, 'https://maps.googleapis.com/maps/api/geocode/json'):
             bounds = result['geometry']['viewport']
             yield (bounds['northeast']['lat'], bounds['northeast']['lng'],
                    bounds['southwest']['lat'], bounds['southwest']['lng'],
                    result['formatted_address'])
 
     def search_terms(self):
-        widget = QtWidgets.QLabel(
-            translate('MapTabGoogle', 'Search and altitude lookup')
-            + '\npowered by Google')
+        widget = Label(translate(
+            'MapTabGoogle', 'Search and altitude lookup powered by Google',
+            'Do not translate "powered by Google"'), lines=2)
         widget.setAlignment(Qt.AlignRight)
         scale_font(widget, 80)
         return [widget]
