@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2019-20  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2019-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -16,23 +16,21 @@
 ##  along with this program.  If not, see
 ##  <http://www.gnu.org/licenses/>.
 
-from __future__ import unicode_literals
-
 import logging
 import os
-import six
 import urllib
 
 import requests
 from requests_oauthlib import OAuth2Session
 
-from photini.configstore import key_store
-from photini.pyqt import catch_all, QtCore, QtSignal, QtSlot, QtWidgets
+from photini.pyqt import (
+    catch_all, execute, QtCore, QtSignal, QtSlot, QtWidgets, width_for_text)
 from photini.uploader import PhotiniUploader, UploaderSession
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
 
+# Google Photos API: https://developers.google.com/photos/library/reference/rest
 
 class GooglePhotosSession(UploaderSession):
     name       = 'googlephotos'
@@ -40,10 +38,6 @@ class GooglePhotosSession(UploaderSession):
     photos_url = 'https://photoslibrary.googleapis.com/'
     scope      = ('profile', 'https://www.googleapis.com/auth/photoslibrary')
 
-    def __init__(self, *arg, **kwds):
-        super(GooglePhotosSession, self).__init__(*arg, **kwds)
-        self.api = None
-
     def open_connection(self, token=None):
         self.cached_data = {}
         refresh_token = self.get_password()
@@ -56,15 +50,14 @@ class GooglePhotosSession(UploaderSession):
                 'refresh_token': refresh_token,
                 'expires_in'   : -30,
                 }
-        client_id = key_store.get('googlephotos', 'client_id')
-        client_secret = key_store.get('googlephotos', 'client_secret')
         auto_refresh_kwargs = {
-            'client_id'    : client_id,
-            'client_secret': client_secret,
+            'client_id'    : self.client_id,
+            'client_secret': self.client_secret,
             }
         token_url = self.oauth_url + 'v4/token'
         self.api = OAuth2Session(
-            client_id=client_id, token=token, token_updater=self.save_token,
+            client_id=self.client_id, token=token,
+            token_updater=self.save_token,
             auto_refresh_kwargs=auto_refresh_kwargs,
             auto_refresh_url=token_url)
         if token['expires_in'] < 0:
@@ -73,12 +66,6 @@ class GooglePhotosSession(UploaderSession):
         self.connection_changed.emit(self.api.authorized)
         return self.api.authorized
 
-    def close_connection(self):
-        self.connection_changed.emit(False)
-        if self.api:
-            self.api.close()
-            self.api = None
-
     def check_response(self, rsp, decode=True):
         if rsp.status_code != 200:
             logger.error('HTTP error %d', rsp.status_code)
@@ -92,8 +79,8 @@ class GooglePhotosSession(UploaderSession):
         while len(code_verifier) < 43:
             code_verifier += OAuth2Session().new_state()
         self.auth_params = {
-            'client_id'    : key_store.get('googlephotos', 'client_id'),
-            'client_secret': key_store.get('googlephotos', 'client_secret'),
+            'client_id'    : self.client_id,
+            'client_secret': self.client_secret,
             'code_verifier': code_verifier,
             'redirect_uri' : redirect_uri,
             }
@@ -181,6 +168,7 @@ class GooglePhotosSession(UploaderSession):
         upload_url = rsp.headers['X-Goog-Upload-URL']
         chunk_size = int(rsp.headers['X-Goog-Upload-Chunk-Granularity'])
         # 2/ upload data in chunks, size set by google
+        self.upload_progress.emit({'busy': False})
         headers = {'X-Goog-Upload-Command': 'upload'}
         offset = 0
         while offset < fileobj.len:
@@ -190,12 +178,13 @@ class GooglePhotosSession(UploaderSession):
             if offset >= fileobj.len:
                 headers['X-Goog-Upload-Command'] = 'upload, finalize'
             rsp = self.api.post(upload_url, headers=headers, data=chunk)
+            self.upload_progress.emit({'value': offset * 100 // fileobj.len})
             rsp = self.check_response(rsp, decode=False)
             if not rsp:
                 break
             if rsp.text:
                 upload_token = rsp.text
-        fileobj._callback(100)
+        self.upload_progress.emit({'busy': True})
         if not upload_token:
             return 'no upload token received'
         # 3/ convert uploaded bytes to a media item
@@ -225,45 +214,39 @@ class GooglePhotosSession(UploaderSession):
         return ''
 
 
-class GoogleUploadConfig(QtWidgets.QWidget):
-    new_set = QtSignal()
+class TabWidget(PhotiniUploader):
+    logger = logger
+    session_factory = GooglePhotosSession
 
-    def __init__(self, *arg, **kw):
-        super(GoogleUploadConfig, self).__init__(*arg, **kw)
-        self.setLayout(QtWidgets.QGridLayout())
-        self.layout().setContentsMargins(0, 0, 0, 0)
+    @staticmethod
+    def tab_name():
+        return translate('GooglePhotosTab', 'Google &Photos upload')
+
+    def config_columns(self):
+        self.service_name = translate('GooglePhotosTab', 'Google Photos')
+        ## first column
+        column = QtWidgets.QGridLayout()
+        column.setContentsMargins(0, 0, 0, 0)
         # create new set
+        group = QtWidgets.QGroupBox()
+        group.setMinimumWidth(width_for_text(group, 'x' * 23))
+        column.addWidget(group, 0, 0)
         new_set_button = QtWidgets.QPushButton(
             translate('GooglePhotosTab', 'New album'))
         new_set_button.clicked.connect(self.new_set)
-        self.layout().addWidget(new_set_button, 2, 1)
-        # list of sets widget
-        sets_group = QtWidgets.QGroupBox(
-            translate('GooglePhotosTab', 'Add to albums'))
-        sets_group.setLayout(QtWidgets.QVBoxLayout())
-        scrollarea = QtWidgets.QScrollArea()
-        scrollarea.setFrameStyle(QtWidgets.QFrame.NoFrame)
-        scrollarea.setStyleSheet("QScrollArea { background-color: transparent }")
-        self.sets_widget = QtWidgets.QWidget()
-        self.sets_widget.setLayout(QtWidgets.QVBoxLayout())
-        self.sets_widget.layout().setSpacing(0)
-        self.sets_widget.layout().setSizeConstraint(
-            QtWidgets.QLayout.SetMinAndMaxSize)
-        scrollarea.setWidget(self.sets_widget)
-        self.sets_widget.setAutoFillBackground(False)
-        sets_group.layout().addWidget(scrollarea)
-        self.layout().addWidget(sets_group, 0, 2, 3, 1)
-        self.layout().setColumnStretch(2, 1)
+        column.addWidget(new_set_button, 1, 0)
+        column.setRowStretch(0, 1)
+        yield column
 
-    def clear_sets(self):
-        for child in self.sets_widget.children():
+    def clear_albums(self):
+        for child in self.widget['albums'].children():
             if child.isWidgetType():
-                self.sets_widget.layout().removeWidget(child)
+                self.widget['albums'].layout().removeWidget(child)
                 child.setParent(None)
 
     def checked_albums(self):
         result = []
-        for child in self.sets_widget.children():
+        for child in self.widget['albums'].children():
             if child.isWidgetType() and child.isChecked():
                 result.append(child.property('id'))
         return result
@@ -273,29 +256,18 @@ class GoogleUploadConfig(QtWidgets.QWidget):
         widget.setProperty('id', album['id'])
         widget.setEnabled(album['isWriteable'])
         if index >= 0:
-            self.sets_widget.layout().insertWidget(index, widget)
+            self.widget['albums'].layout().insertWidget(index, widget)
         else:
-            self.sets_widget.layout().addWidget(widget)
+            self.widget['albums'].layout().addWidget(widget)
         return widget
 
-
-class TabWidget(PhotiniUploader):
-    session_factory = GooglePhotosSession
-
-    @staticmethod
-    def tab_name():
-        return translate('GooglePhotosTab', 'Google &Photos upload')
-
-    def __init__(self, *arg, **kw):
-        self.service_name = translate('GooglePhotosTab', 'Google Photos')
-        self.upload_config = GoogleUploadConfig()
-        super(TabWidget, self).__init__(self.upload_config, *arg, **kw)
-        self.upload_config.new_set.connect(self.new_set)
-        self.image_types = {
-            'accepted': ('image/gif', 'image/jpeg', 'image/png',
-                         'video/mp4', 'video/quicktime', 'video/riff'),
-            'rejected': ('image/x-canon-cr2',),
-            }
+    def accepted_file_type(self, file_type):
+        if file_type.startswith('video'):
+            # Google seems to accept any video format
+            return True
+        return file_type in ('image/gif', 'image/heic', 'image/jpeg',
+                             'image/png', 'image/tiff', 'image/webp',
+                             'image/x-ms-bmp')
 
     def get_conversion_function(self, image, params):
         convert = super(
@@ -309,24 +281,19 @@ class TabWidget(PhotiniUploader):
         dialog = QtWidgets.QMessageBox(parent=self)
         dialog.setWindowTitle(
             translate('GooglePhotosTab', 'Photini: large file'))
-        dialog.setText(
-            translate('GooglePhotosTab', '<h3>Large file.</h3>'))
+        dialog.setText('<h3>{}</h3>'.format(
+            translate('GooglePhotosTab', 'Large file.')))
         dialog.setInformativeText(
             translate('GooglePhotosTab',
-                      'File "{0}" is over 25MB. Remember that Photini '
+                      'File "{0}" is over 25 MB. Remember that Photini '
                       'uploads count towards storage in your Google Account. '
                       'Upload it anyway?').format(os.path.basename(image.path)))
         dialog.setIcon(QtWidgets.QMessageBox.Warning)
         dialog.setStandardButtons(
             QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.Ignore)
-        dialog.exec_()
+        execute(dialog)
         return 'omit'
 
-    def show_album_list(self, albums):
-        self.upload_config.clear_sets()
-        for album in albums:
-            self.upload_config.add_album(album)
-
     def get_upload_params(self, image):
         title = image.metadata.title
         description = image.metadata.description
@@ -338,7 +305,7 @@ class TabWidget(PhotiniUploader):
             description = image.path
         params = {
             'description': description,
-            'albums'     : self.upload_config.checked_albums(),
+            'albums'     : self.checked_albums(),
             }
         return params
 
@@ -353,5 +320,5 @@ class TabWidget(PhotiniUploader):
         album = self.session.new_album(title)
         if not album:
             return
-        widget = self.upload_config.add_album(album, index=0)
+        widget = self.add_album(album, index=0)
         widget.setChecked(True)
