@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -17,14 +16,10 @@
 ##  along with this program.  If not, see
 ##  <http://www.gnu.org/licenses/>.
 
-from __future__ import unicode_literals
-
-import six
 from datetime import datetime
+import io
 import logging
 import os
-from six import BytesIO
-from six.moves.urllib.parse import unquote
 
 try:
     import PIL.Image as PIL
@@ -33,9 +28,9 @@ except ImportError:
 
 from photini.ffmpeg import FFmpeg
 from photini.metadata import Metadata
+from photini.pyqt import *
 from photini.pyqt import (
-    Busy, catch_all, image_types, Qt, QtCore, QtGui, QtSignal, QtSlot,
-    QtWidgets, qt_version_info, scale_font, set_symbol_font, video_types)
+    image_types, qt_version_info, set_symbol_font, video_types)
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
@@ -43,17 +38,15 @@ DRAG_MIMETYPE = 'application/x-photini-image'
 
 
 class Image(QtWidgets.QFrame):
-    def __init__(self, path, image_list, thumb_size=80, *arg, **kw):
+    def __init__(self, path, image_list, thumb_size=4, *arg, **kw):
         super(Image, self).__init__(*arg, **kw)
         self.app = QtWidgets.QApplication.instance()
         self.path = path
         self.image_list = image_list
         self.name, ext = os.path.splitext(os.path.basename(self.path))
         self.selected = False
-        self.thumb_size = thumb_size
         # read metadata
-        self.metadata = Metadata(self.path, notify=self.show_status,
-                                 utf_safe=self.app.options.utf_safe)
+        self.metadata = Metadata(self.path, notify=self.show_status)
         self.file_times = (os.path.getatime(self.path),
                            os.path.getmtime(self.path))
         # set file type
@@ -63,9 +56,10 @@ class Image(QtWidgets.QFrame):
         layout.setSpacing(0)
         layout.setContentsMargins(3, 3, 3, 3)
         self.setLayout(layout)
-        self.setToolTip(self.path)
+        self.setToolTip('<p>' + self.path + '</p>')
         # label to display image
         self.image = QtWidgets.QLabel()
+        self.image.setWordWrap(True)
         self.image.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
         layout.addWidget(self.image, 0, 0, 1, 2)
         # label to display file name
@@ -83,7 +77,7 @@ class Image(QtWidgets.QFrame):
         self.setObjectName("thumbnail")
         self.set_selected(False)
         self.show_status(False)
-        self._set_thumb_size(self.thumb_size)
+        self._set_thumb_size(thumb_size)
 
     def reload_metadata(self):
         self.metadata = Metadata(self.path, notify=self.show_status)
@@ -110,25 +104,25 @@ class Image(QtWidgets.QFrame):
     def regenerate_thumbnail(self):
         # DCF spec says thumbnail must be 160 x 120, so other aspect
         # ratios are padded with black
-        # first try using FFmpeg to make thumbnail
-        data = self.make_thumb_ffmpeg()
-        if data:
-            self.metadata.thumbnail = {'data': data}
-            return True
-        # use PIL or Qt
+        # try using PIL first, good quality and quick
         qt_im = self.get_qt_image()
-        if not qt_im or qt_im.isNull():
-            return False
-        if PIL:
+        if qt_im and PIL:
             data = self.make_thumb_PIL(qt_im)
             if data:
                 self.metadata.thumbnail = {'data': data}
                 return True
-        qt_im = self.make_thumb_Qt(qt_im)
-        if not qt_im or qt_im.isNull():
-            return False
-        self.metadata.thumbnail = {'image': qt_im}
-        return True
+        # next try using FFmpeg, good quality but slower
+        data = self.make_thumb_ffmpeg()
+        if data:
+            self.metadata.thumbnail = {'data': data}
+            return True
+        # lastly use Qt, quick but not high quality
+        if qt_im:
+            qt_im = self.make_thumb_Qt(qt_im)
+            if qt_im:
+                self.metadata.thumbnail = {'image': qt_im}
+                return True
+        return False
 
     def make_thumb_ffmpeg(self):
         # get input dimensions
@@ -168,8 +162,7 @@ class Image(QtWidgets.QFrame):
         reader.setAutoTransform(False)
         qt_im = reader.read()
         if not qt_im or qt_im.isNull():
-            logger.error('Cannot read %s image data from %s',
-                         self.file_type, self.path)
+            logger.error('Image read: %s: %s', self.path, reader.errorString())
             return None
         w = qt_im.width()
         h = qt_im.height()
@@ -184,21 +177,15 @@ class Image(QtWidgets.QFrame):
         if w >= h:
             new_h = int(0.5 + (float(w * 3) / 4.0))
             new_w = int(0.5 + (float(h * 4) / 3.0))
-            if new_h > h:
-                pad = (new_h - h) // 2
-                qt_im = qt_im.copy(0, -pad, w, new_h)
-            elif new_w > w:
-                pad = (new_w - w) // 2
-                qt_im = qt_im.copy(-pad, 0, new_w, h)
         else:
             new_h = int(0.5 + (float(w * 4) / 3.0))
             new_w = int(0.5 + (float(h * 3) / 4.0))
-            if new_w > w:
-                pad = (new_w - w) // 2
-                qt_im = qt_im.copy(-pad, 0, new_w, h)
-            elif new_h > h:
-                pad = (new_h - h) // 2
-                qt_im = qt_im.copy(0, -pad, w, new_h)
+        if new_w > w:
+            pad = (new_w - w) // 2
+            qt_im = qt_im.copy(-pad, 0, new_w, h)
+        elif new_h > h:
+            pad = (new_h - h) // 2
+            qt_im = qt_im.copy(0, -pad, w, new_h)
         return qt_im
 
     def make_thumb_PIL(self, qt_im):
@@ -209,7 +196,7 @@ class Image(QtWidgets.QFrame):
         buf = QtCore.QBuffer()
         buf.open(buf.WriteOnly)
         qt_im.save(buf, 'PPM')
-        data = BytesIO(buf.data().data())
+        data = io.BytesIO(buf.data().data())
         try:
             pil_im = PIL.open(data)
         except Exception as ex:
@@ -218,7 +205,7 @@ class Image(QtWidgets.QFrame):
         # scale PIL image
         pil_im.thumbnail((w, h), PIL.ANTIALIAS)
         # save image to memory
-        data = BytesIO()
+        data = io.BytesIO()
         pil_im.save(data, 'JPEG')
         return data.getvalue()
 
@@ -233,33 +220,39 @@ class Image(QtWidgets.QFrame):
     def contextMenuEvent(self, event):
         menu = QtWidgets.QMenu(self)
         self.image_list.add_selected_actions(menu)
-        menu.exec_(event.globalPos())
+        execute(menu, event.globalPos())
 
     @catch_all
     def mousePressEvent(self, event):
         if event.button() == Qt.LeftButton:
-            self.drag_start_pos = event.pos()
+            if qt_version_info >= (6, 0):
+                self.drag_start_pos = event.position()
+            else:
+                self.drag_start_pos = event.pos()
+
+    @catch_all
+    def mouseReleaseEvent(self, event):
         if event.modifiers() == Qt.ControlModifier:
             self.image_list.select_image(self, multiple_selection=True)
         elif event.modifiers() == Qt.ShiftModifier:
             self.image_list.select_image(self, extend_selection=True)
-        elif not self.get_selected():
-            # don't clear selection in case we're about to drag
-            self.image_list.select_image(self)
-
-    @catch_all
-    def mouseReleaseEvent(self, event):
-        if event.modifiers() not in (Qt.ControlModifier, Qt.ShiftModifier):
-            # clear any multiple selection
+        else:
             self.image_list.select_image(self)
 
     @catch_all
     def mouseMoveEvent(self, event):
         if not self.image_list.drag_icon:
             return
-        if ((event.pos() - self.drag_start_pos).manhattanLength() <
+        if qt_version_info >= (6, 0):
+            pos = event.position()
+        else:
+            pos = event.pos()
+        if ((pos - self.drag_start_pos).manhattanLength() <
                                     QtWidgets.QApplication.startDragDistance()):
             return
+        if not self.get_selected():
+            # user has started dragging an unselected image
+            self.image_list.select_image(self, emit_selection=False)
         paths = []
         for image in self.image_list.get_selected_images():
             paths.append(image.path)
@@ -293,7 +286,9 @@ class Image(QtWidgets.QFrame):
         mimeData = QtCore.QMimeData()
         mimeData.setData(DRAG_MIMETYPE, repr(paths).encode('utf-8'))
         drag.setMimeData(mimeData)
-        dropAction = drag.exec_(Qt.CopyAction)
+        if execute(drag, Qt.CopyAction) == Qt.IgnoreAction:
+            # image wasn't dragged to map
+            self.image_list.emit_selection()
 
     @catch_all
     def mouseDoubleClickEvent(self, event):
@@ -303,10 +298,10 @@ class Image(QtWidgets.QFrame):
         status = ''
         # set 'geotagged' status
         if self.metadata.latlong:
-            status += six.unichr(0x2690)
+            status += chr(0x2690)
         # set 'unsaved' status
         if changed:
-            status += six.unichr(0x26A1)
+            status += chr(0x26A1)
         self.status.setText(status)
         self._elide_name()
         if changed:
@@ -315,12 +310,12 @@ class Image(QtWidgets.QFrame):
     def _elide_name(self):
         self.status.adjustSize()
         elided_name = self.label.fontMetrics().elidedText(
-            self.name, Qt.ElideLeft, self.thumb_size - self.status.width())
+            self.name, Qt.ElideLeft, self.image.width() - self.status.width())
         self.label.setText(elided_name)
 
     def _set_thumb_size(self, thumb_size):
-        self.thumb_size = thumb_size
-        self.image.setFixedSize(self.thumb_size, self.thumb_size)
+        width = width_for_text(self.label, 'X' * thumb_size * 20) // 6
+        self.image.setFixedSize(width, width)
         self._elide_name()
 
     def set_thumb_size(self, thumb_size):
@@ -330,12 +325,14 @@ class Image(QtWidgets.QFrame):
     def load_thumbnail(self):
         image = self.metadata.thumbnail and self.metadata.thumbnail['image']
         if not image:
-            self.image.setText(translate('ImageList', 'No\nthumbnail\nin file'))
+            self.image.setText(wrap_text(
+                self.image, translate('ImageList', 'No thumbnail in file'),
+                lines=4))
             return
         pixmap = QtGui.QPixmap.fromImage(image)
         pixmap = self.transform(pixmap, self.metadata.orientation)
         self.image.setPixmap(
-            pixmap.scaled(self.thumb_size, self.thumb_size,
+            pixmap.scaled(self.image.width(), self.image.height(),
                           Qt.KeepAspectRatio, Qt.SmoothTransformation))
 
     def set_selected(self, value):
@@ -351,6 +348,7 @@ class Image(QtWidgets.QFrame):
 
 class ScrollArea(QtWidgets.QScrollArea):
     dropped_images = QtSignal(list)
+    multi_row_changed = QtSignal()
 
     def __init__(self, parent=None):
         super(ScrollArea, self).__init__(parent)
@@ -382,6 +380,7 @@ class ScrollArea(QtWidgets.QScrollArea):
         else:
             self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
             self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+        self.multi_row_changed.emit()
 
     def ensureWidgetVisible(self, widget):
         left, top, right, bottom = self.thumbs.getContentsMargins()
@@ -425,7 +424,7 @@ class ThumbsLayout(QtWidgets.QLayout):
         self.scroll_area = scroll_area
         self.item_list = []
         self.viewport_size = QtCore.QSize()
-        self._do_layout(QtCore.QPoint(0, 0))
+        self._do_layout()
 
     def addItem(self, item):
         self.item_list.append(item)
@@ -457,7 +456,7 @@ class ThumbsLayout(QtWidgets.QLayout):
 
     def setGeometry(self, rect):
         super(ThumbsLayout, self).setGeometry(rect)
-        self._do_layout(rect.topLeft())
+        self._do_layout(rect)
 
     def sizeHint(self):
         return self.size_hint
@@ -467,9 +466,9 @@ class ThumbsLayout(QtWidgets.QLayout):
 
     def set_viewport_size(self, size):
         self.viewport_size = size
-        self._do_layout(QtCore.QPoint(0, 0))
+        self._do_layout()
 
-    def _do_layout(self, origin):
+    def _do_layout(self, rect=None):
         left, top, right, bottom = self.getContentsMargins()
         width_hint = left + right
         height_hint = top + bottom
@@ -488,10 +487,14 @@ class ThumbsLayout(QtWidgets.QLayout):
             width_hint += columns * item_w
             height_hint += rows * item_h
         self.size_hint = QtCore.QSize(width_hint, height_hint)
-        if not self.item_list:
+        if not (rect and self.item_list):
             return
-        x = origin.x() + left
-        y = origin.y() + top
+        if QtWidgets.QApplication.isRightToLeft():
+            x = rect.right() - right - item_w
+            item_w = -item_w
+        else:
+            x = rect.left() + left
+        y = rect.top() + top
         for n, item in enumerate(self.item_list):
             i, j = n % columns, n // columns
             item.setGeometry(QtCore.QRect(
@@ -512,56 +515,59 @@ class ImageList(QtWidgets.QWidget):
         self.images = []
         self.last_selected = None
         self.selection_anchor = None
-        self.thumb_size = int(
-            self.app.config_store.get('controls', 'thumb_size', '80'))
-        layout = QtWidgets.QGridLayout()
-        layout.setSpacing(0)
-        layout.setRowStretch(0, 1)
-        layout.setColumnStretch(3, 1)
-        self.setLayout(layout)
-        layout.setContentsMargins(0, 0, 0, 0)
+        self.thumb_size = self.app.config_store.get('controls', 'thumb_size', 4)
+        if self.thumb_size > 20:
+            # old config, in pixels
+            self.thumb_size = self.thumb_size // 20
         # thumbnail display
+        self.setLayout(QtWidgets.QVBoxLayout())
+        self.layout().setContentsMargins(0, 0, 0, 0)
         self.scroll_area = ScrollArea()
         self.scroll_area.dropped_images.connect(self.open_file_list)
-        layout.addWidget(self.scroll_area, 0, 0, 1, 6)
-        QtWidgets.QShortcut(QtGui.QKeySequence.MoveToPreviousChar,
-                        self.scroll_area, self.move_to_prev_thumb)
-        QtWidgets.QShortcut(QtGui.QKeySequence.MoveToNextChar,
-                        self.scroll_area, self.move_to_next_thumb)
-        QtWidgets.QShortcut(QtGui.QKeySequence.MoveToStartOfLine,
-                        self.scroll_area, self.move_to_first_thumb)
-        QtWidgets.QShortcut(QtGui.QKeySequence.MoveToEndOfLine,
-                        self.scroll_area, self.move_to_last_thumb)
-        QtWidgets.QShortcut(QtGui.QKeySequence.SelectPreviousChar,
-                        self.scroll_area, self.select_prev_thumb)
-        QtWidgets.QShortcut(QtGui.QKeySequence.SelectNextChar,
-                        self.scroll_area, self.select_next_thumb)
-        QtWidgets.QShortcut(QtGui.QKeySequence.SelectAll,
-                        self.scroll_area, self.select_all)
+        self.scroll_area.multi_row_changed.connect(
+            self._ensure_selected_visible)
+        self.layout().addWidget(self.scroll_area)
+        QtGui2.QShortcut(QtGui.QKeySequence.MoveToPreviousChar,
+                         self.scroll_area, self.move_to_prev_thumb)
+        QtGui2.QShortcut(QtGui.QKeySequence.MoveToNextChar,
+                         self.scroll_area, self.move_to_next_thumb)
+        QtGui2.QShortcut(QtGui.QKeySequence.MoveToStartOfLine,
+                         self.scroll_area, self.move_to_first_thumb)
+        QtGui2.QShortcut(QtGui.QKeySequence.MoveToEndOfLine,
+                         self.scroll_area, self.move_to_last_thumb)
+        QtGui2.QShortcut(QtGui.QKeySequence.SelectPreviousChar,
+                         self.scroll_area, self.select_prev_thumb)
+        QtGui2.QShortcut(QtGui.QKeySequence.SelectNextChar,
+                         self.scroll_area, self.select_next_thumb)
+        QtGui2.QShortcut(QtGui.QKeySequence.SelectAll,
+                         self.scroll_area, self.select_all)
         # sort key selector
-        layout.addWidget(QtWidgets.QLabel(self.tr('sort by: ')), 1, 0)
+        bottom_bar = QtWidgets.QHBoxLayout()
+        self.layout().addLayout(bottom_bar)
+        bottom_bar.addWidget(QtWidgets.QLabel(self.tr('Sort by')))
         self.sort_name = QtWidgets.QRadioButton(self.tr('file name'))
         self.sort_name.clicked.connect(self._new_sort_order)
-        layout.addWidget(self.sort_name, 1, 1)
+        bottom_bar.addWidget(self.sort_name)
         self.sort_date = QtWidgets.QRadioButton(self.tr('date taken'))
-        layout.addWidget(self.sort_date, 1, 2)
         self.sort_date.clicked.connect(self._new_sort_order)
-        if eval(self.app.config_store.get('controls', 'sort_date', 'False')):
+        bottom_bar.addWidget(self.sort_date)
+        if self.app.config_store.get('controls', 'sort_date', False):
             self.sort_date.setChecked(True)
         else:
             self.sort_name.setChecked(True)
         # size selector
-        layout.addWidget(QtWidgets.QLabel(self.tr('thumbnail size: ')), 1, 4)
+        bottom_bar.addStretch(1)
+        bottom_bar.addWidget(QtWidgets.QLabel(self.tr('Thumbnail size')))
         self.size_slider = QtWidgets.QSlider(Qt.Horizontal)
         self.size_slider.setTracking(False)
         self.size_slider.setRange(4, 9)
         self.size_slider.setPageStep(1)
-        self.size_slider.setValue(self.thumb_size // 20)
+        self.size_slider.setValue(self.thumb_size)
         self.size_slider.setTickPosition(QtWidgets.QSlider.TicksBelow)
-        width = self.size_slider.sizeHint().width()
-        self.size_slider.setMinimumWidth(width * 7 // 4)
+        self.size_slider.setMinimumWidth(
+            width_for_text(self.size_slider, 'x' * 20))
         self.size_slider.valueChanged.connect(self._new_thumb_size)
-        layout.addWidget(self.size_slider, 1, 5)
+        bottom_bar.addWidget(self.size_slider)
 
     def set_drag_to_map(self, icon, hotspot=None):
         self.drag_icon = icon
@@ -595,41 +601,57 @@ class ImageList(QtWidgets.QWidget):
                 ' '.join(['*.' + x for x in image_types()]),
                 ' '.join(['*.' + x for x in video_types()]))
             ]
-        if eval(self.app.config_store.get('pyqt', 'native_dialog', 'True')):
-            pass
-        elif qt_version_info >= (5, 0):
+        if not self.app.config_store.get('pyqt', 'native_dialog', True):
             args += [None, QtWidgets.QFileDialog.DontUseNativeDialog]
-        else:
-            args += [QtWidgets.QFileDialog.DontUseNativeDialog]
         path_list = QtWidgets.QFileDialog.getOpenFileNames(*args)
-        if qt_version_info >= (5, 0):
-            path_list = path_list[0]
+        path_list = path_list[0]
         if not path_list:
             return
-        # work around for Qt bug 33992
-        # https://bugreports.qt-project.org/browse/QTBUG-33992
-        if qt_version_info in ((4, 8, 4), (4, 8, 5)):
-            path_list = list(map(unquote, path_list))
         self.open_file_list(path_list)
 
     @QtSlot(list)
     @catch_all
-    def open_file_list(self, path_list):
+    def open_file_list(self, path_list, top_level=True):
+        last_path = None
         with Busy():
             for path in path_list:
-                self.open_file(path)
-        self.done_opening(path_list[-1])
+                if os.path.basename(path).startswith('.'):
+                    # don't open .directory or .thumbs
+                    continue
+                if os.path.isdir(path):
+                    last_path = self.open_file_list(
+                        [os.path.join(path, x) for x in os.listdir(path)],
+                        top_level=False) or last_path
+                elif self.open_file(path):
+                    last_path = path
+        if top_level and last_path:
+            self.done_opening(last_path)
+        return last_path
 
     def open_file(self, path):
         path = os.path.abspath(path)
+        base, ext = os.path.splitext(path)
+        if ext.lower() == '.xmp':
+            if os.path.isfile(base):
+                path = base
+            else:
+                dir_name = os.path.dirname(path)
+                for file in os.listdir(dir_name):
+                    path = os.path.join(dir_name, file)
+                    b, e = os.path.splitext(path)
+                    if b == base and e.lower() != '.xmp':
+                        break
+                else:
+                    return False
         if not os.path.isfile(path):
-            return
+            return False
         if self.get_image(path):
             # already opened this path
-            return
+            return True
         image = Image(path, self, thumb_size=self.thumb_size)
         self.images.append(image)
         self.show_thumbnail(image)
+        return True
 
     def done_opening(self, path):
         self.app.config_store.set(
@@ -658,9 +680,15 @@ class ImageList(QtWidgets.QWidget):
         self._sort_thumbnails()
         self.sort_order_changed.emit()
 
+    @QtSlot()
+    @catch_all
+    def _ensure_selected_visible(self):
+        if self.last_selected:
+            self.scroll_area.ensureWidgetVisible(self.last_selected)
+
     def _sort_thumbnails(self):
         sort_date = self.sort_date.isChecked()
-        self.app.config_store.set('controls', 'sort_date', str(sort_date))
+        self.app.config_store.set('controls', 'sort_date', sort_date)
         with Busy():
             if sort_date:
                 self.images.sort(key=self._date_key)
@@ -730,8 +758,9 @@ class ImageList(QtWidgets.QWidget):
     def diff_selected_metadata(self):
         dialog = QtWidgets.QDialog(parent=self)
         dialog.setLayout(QtWidgets.QVBoxLayout())
-        dialog.setFixedSize(min(800, self.window().width()),
-                            min(400, self.window().height()))
+        width = width_for_text(dialog, 'x' * 120)
+        dialog.setFixedSize(min(width, self.window().width()),
+                            min(width // 2, self.window().height()))
         table = QtWidgets.QTableWidget()
         table.setColumnCount(3)
         table.setHorizontalHeaderLabels([translate('ImageList', 'new value'),
@@ -761,10 +790,10 @@ class ImageList(QtWidgets.QWidget):
             new_md = image.metadata
             old_md = Metadata(image.path)
             for key in ('title', 'description', 'keywords', 'rating',
-                        'copyright', 'creator',
+                        'creator', 'creator_title', 'credit_line', 'copyright',
+                        'rights', 'instructions', 'contact_info',
                         'date_taken', 'date_digitised', 'date_modified',
-                        'orientation', 'camera_model',
-                        'lens_model', 'lens_spec',
+                        'orientation', 'camera_model', 'lens_model',
                         'focal_length', 'focal_length_35', 'aperture',
                         'latlong', 'altitude',
                         'location_taken', 'location_shown',
@@ -772,7 +801,7 @@ class ImageList(QtWidgets.QWidget):
                 values = getattr(new_md, key), getattr(old_md, key)
                 if values[0] == values[1]:
                     continue
-                values = [six.text_type(x or '') for x in values]
+                values = [str(x or '') for x in values]
                 table.setRowCount(row + 1)
                 for n, value in enumerate(values):
                     item = QtWidgets.QTableWidgetItem(value)
@@ -790,7 +819,8 @@ class ImageList(QtWidgets.QWidget):
             table.resizeRowsToContents()
             if position:
                 dialog.move(position)
-            if dialog.exec_() != QtWidgets.QDialog.Accepted:
+            result = execute(dialog)
+            if result != QtWidgets.QDialog.Accepted:
                 return
             position = dialog.pos()
             undo_all = True
@@ -812,6 +842,19 @@ class ImageList(QtWidgets.QWidget):
             for image in self.get_selected_images():
                 if image.regenerate_thumbnail():
                     image.load_thumbnail()
+                    self.app.processEvents()
+
+    @QtSlot()
+    @catch_all
+    def fix_missing_thumbs(self):
+        with Busy():
+            for image in self.get_images():
+                thumb = image.metadata.thumbnail
+                if not thumb or not thumb['image']:
+                    if image.regenerate_thumbnail():
+                        image.load_thumbnail()
+                        self.app.processEvents()
+        self.image_list_changed.emit()
 
     @QtSlot()
     @catch_all
@@ -851,33 +894,37 @@ class ImageList(QtWidgets.QWidget):
         self._save_files(self.images)
 
     def _save_files(self, images=[]):
-        if_mode = eval(self.app.config_store.get('files', 'image', 'True'))
+        self._flush_editing()
+        if_mode = self.app.config_store.get('files', 'image', True)
         sc_mode = self.app.config_store.get('files', 'sidecar', 'auto')
-        force_iptc = eval(
-            self.app.config_store.get('files', 'force_iptc', 'False'))
-        keep_time = eval(
-            self.app.config_store.get('files', 'preserve_timestamps', 'False'))
+        iptc_mode = self.app.config_store.get('files', 'iptc_iim', 'preserve')
+        keep_time = self.app.config_store.get(
+            'files', 'preserve_timestamps', 'now')
+        if isinstance(keep_time, bool):
+            # old config format
+            keep_time = ('now', 'keep')[keep_time]
         if not images:
             images = self.images
         with Busy():
             for image in images:
-                if keep_time:
+                if keep_time == 'taken' and image.metadata.date_taken:
+                    file_times = (
+                        image.file_times[0],
+                        image.metadata.date_taken['datetime'].timestamp())
+                elif keep_time == 'keep':
                     file_times = image.file_times
                 else:
                     file_times = None
                 image.metadata.save(
                     if_mode=if_mode, sc_mode=sc_mode,
-                    force_iptc=force_iptc, file_times=file_times)
-        unsaved = False
-        for image in self.images:
-            if image.metadata.changed():
-                unsaved = True
-                break
+                    iptc_mode=iptc_mode, file_times=file_times)
+        unsaved = any([image.metadata.changed() for image in self.images])
         self.new_metadata.emit(unsaved)
 
     def unsaved_files_dialog(
             self, all_files=False, with_cancel=True, with_discard=True):
         """Return true if OK to continue with close or quit or whatever"""
+        self._flush_editing()
         for image in self.images:
             if image.metadata.changed() and (all_files or image.selected):
                 break
@@ -885,7 +932,8 @@ class ImageList(QtWidgets.QWidget):
             return True
         dialog = QtWidgets.QMessageBox(parent=self)
         dialog.setWindowTitle(self.tr('Photini: unsaved data'))
-        dialog.setText(self.tr('<h3>Some images have unsaved metadata.</h3>'))
+        dialog.setText('<h3>{}</h3>'.format(
+            self.tr('Some images have unsaved metadata.')))
         dialog.setInformativeText(self.tr('Do you want to save your changes?'))
         dialog.setIcon(QtWidgets.QMessageBox.Warning)
         buttons = QtWidgets.QMessageBox.Save
@@ -895,12 +943,18 @@ class ImageList(QtWidgets.QWidget):
             buttons |= QtWidgets.QMessageBox.Discard
         dialog.setStandardButtons(buttons)
         dialog.setDefaultButton(QtWidgets.QMessageBox.Save)
-        result = dialog.exec_()
+        result = execute(dialog)
         if result == QtWidgets.QMessageBox.Save:
             self._save_files()
             return True
         return result == QtWidgets.QMessageBox.Discard
 
+    def _flush_editing(self):
+        # finish any editing in progress
+        current_focus = self.app.focusWidget()
+        if current_focus:
+            current_focus.clearFocus()
+
     def get_selected_images(self):
         selection = []
         for image in self.images:
@@ -947,16 +1001,16 @@ class ImageList(QtWidgets.QWidget):
     @QtSlot(int)
     @catch_all
     def _new_thumb_size(self, value):
-        self.thumb_size = value * 20
-        self.app.config_store.set('controls', 'thumb_size', str(self.thumb_size))
+        self.thumb_size = value
+        self.app.config_store.set('controls', 'thumb_size', self.thumb_size)
         for image in self.images:
             image.set_thumb_size(self.thumb_size)
         if self.last_selected:
             self.app.processEvents()
             self.scroll_area.ensureWidgetVisible(self.last_selected)
 
-    def select_image(
-            self, image, extend_selection=False, multiple_selection=False):
+    def select_image(self, image, extend_selection=False,
+                     multiple_selection=False, emit_selection=True):
         self.scroll_area.ensureWidgetVisible(image)
         if extend_selection and self.selection_anchor:
             idx1 = self.images.index(self.selection_anchor)
@@ -974,7 +1028,8 @@ class ImageList(QtWidgets.QWidget):
             image.set_selected(True)
             self.selection_anchor = image
         self.last_selected = image
-        self.emit_selection()
+        if emit_selection:
+            self.emit_selection()
 
     def select_images(self, images):
         self._clear_selection()
