@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -21,7 +21,6 @@ from contextlib import contextmanager
 from datetime import datetime
 import logging
 import os
-import six
 import re
 import shutil
 import sys
@@ -32,10 +31,9 @@ except ImportError:
     gp = None
 
 from photini.metadata import Metadata
-from photini.pyqt import (
-    Busy, catch_all, image_types_lower, Qt, QtCore, QtGui,
-    QtSignal, QtSlot, QtWidgets, qt_version_info, StartStopButton,
-    video_types_lower)
+from photini.pyqt import *
+from photini.pyqt import image_types_lower, video_types_lower
+from photini.widgets import ComboBox, PushButton, StartStopButton
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
@@ -285,8 +283,7 @@ class TabWidget(QtWidgets.QWidget):
         self.config_store = self.app.config_store
         self.image_list = image_list
         self.setLayout(QtWidgets.QGridLayout())
-        form = QtWidgets.QFormLayout()
-        form.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
+        form = FormLayout()
         self.nm = NameMangler()
         self.file_data = {}
         self.file_list = []
@@ -296,8 +293,11 @@ class TabWidget(QtWidgets.QWidget):
         # source selector
         box = QtWidgets.QHBoxLayout()
         box.setContentsMargins(0, 0, 0, 0)
-        self.source_selector = QtWidgets.QComboBox()
+        self.source_selector = ComboBox()
         self.source_selector.currentIndexChanged.connect(self.new_source)
+        self.source_selector.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.source_selector.customContextMenuRequested.connect(
+            self.remove_folder)
         box.addWidget(self.source_selector)
         refresh_button = QtWidgets.QPushButton(
             translate('ImporterTab', 'refresh'))
@@ -305,6 +305,16 @@ class TabWidget(QtWidgets.QWidget):
         box.addWidget(refresh_button)
         box.setStretch(0, 1)
         form.addRow(translate('ImporterTab', 'Source'), box)
+        # update config
+        self.config_store.delete('importer', 'folders')
+        for section in self.config_store.config.sections():
+            if not section.startswith('importer'):
+                continue
+            path_format = self.config_store.get(section, 'path_format')
+            if not (path_format and '(' in path_format):
+                continue
+            path_format = path_format.replace('(', '{').replace(')', '}')
+            self.config_store.set(section, 'path_format', path_format)
         # path format
         self.path_format = QtWidgets.QLineEdit()
         self.path_format.setValidator(PathFormatValidator())
@@ -327,24 +337,22 @@ class TabWidget(QtWidgets.QWidget):
         buttons.addStretch(1)
         self.selected_count = QtWidgets.QLabel()
         buttons.addWidget(self.selected_count)
-        select_all = QtWidgets.QPushButton(
-            translate('ImporterTab', 'Select\nall'))
+        select_all = PushButton(translate('ImporterTab', 'Select all'), lines=2)
         select_all.clicked.connect(self.select_all)
         buttons.addWidget(select_all)
-        select_new = QtWidgets.QPushButton(
-            translate('ImporterTab', 'Select\nnew'))
+        select_new = PushButton(translate('ImporterTab', 'Select new'), lines=2)
         select_new.clicked.connect(self.select_new)
         buttons.addWidget(select_new)
         # copy buttons
         self.move_button = StartStopButton(
-            translate('ImporterTab', 'Move\nphotos'),
-            translate('ImporterTab', 'Stop\nmove'))
+            translate('ImporterTab', 'Move photos'),
+            translate('ImporterTab', 'Stop move'), lines=2)
         self.move_button.click_start.connect(self.move_selected)
         self.move_button.click_stop.connect(self.stop_copy)
         buttons.addWidget(self.move_button)
         self.copy_button = StartStopButton(
-            translate('ImporterTab', 'Copy\nphotos'),
-            translate('ImporterTab', 'Stop\ncopy'))
+            translate('ImporterTab', 'Copy photos'),
+            translate('ImporterTab', 'Stop copy'), lines=2)
         self.copy_button.click_start.connect(self.copy_selected)
         self.copy_button.click_stop.connect(self.stop_copy)
         buttons.addWidget(self.copy_button)
@@ -352,16 +360,10 @@ class TabWidget(QtWidgets.QWidget):
         self.selection_changed()
         # final initialisation
         self.image_list.sort_order_changed.connect(self.sort_file_list)
-        if qt_version_info >= (5, 0):
-            path = QtCore.QStandardPaths.writableLocation(
-                QtCore.QStandardPaths.PicturesLocation)
-        else:
-            path = QtGui.QDesktopServices.storageLocation(
-                QtGui.QDesktopServices.PicturesLocation)
+        path = QtCore.QStandardPaths.writableLocation(
+            QtCore.QStandardPaths.PicturesLocation)
         self.path_format.setText(
             os.path.join(path, '%Y', '%Y_%m_%d', '{name}'))
-        self.refresh()
-        self.list_files()
 
     @QtSlot(int)
     @catch_all
@@ -379,32 +381,61 @@ class TabWidget(QtWidgets.QWidget):
         path_format = self.path_format.text()
         path_format = self.config_store.get(
             self.config_section, 'path_format', path_format)
-        path_format = path_format.replace('(', '{').replace(')', '}')
         self.path_format.setText(path_format)
         self.file_list_widget.clear()
         # allow 100ms for display to update before getting file list
         QtCore.QTimer.singleShot(100, self.list_files)
 
+    @QtSlot(QtCore.QPoint)
+    @catch_all
+    def remove_folder(self, pos):
+        menu = QtWidgets.QMenu()
+        roots = []
+        for section in self.config_store.config.sections():
+            if not section.startswith('importer folder '):
+                continue
+            roots.append((section[16:], self.config_store.get(
+                section, 'last_transfer', datetime.min.isoformat(' '))))
+        roots.sort(key=lambda x: x[1], reverse=True)
+        for root, last_transfer in roots:
+            name = translate('ImporterTab', 'folder: {0}').format(root)
+            action = QtGui2.QAction(
+                translate('TechnicalTab', 'Remove "{}"').format(name),
+                parent=self)
+            action.setData('importer folder ' + root)
+            menu.addAction(action)
+        if menu.isEmpty():
+            return
+        action = execute(menu, self.mapToGlobal(pos))
+        if not action:
+            return
+        self.config_store.remove_section(action.data())
+        self.refresh()
+
     def add_folder(self):
-        folders = eval(self.config_store.get('importer', 'folders', '[]'))
-        if folders:
-            directory = folders[0]
-        else:
-            directory = ''
+        directory = ''
+        for idx in range(self.source_selector.count()):
+            item_data = self.source_selector.itemData(idx)
+            if callable(item_data):
+                continue
+            source, section = item_data
+            if section.startswith('importer folder '):
+                directory = section[16:]
+                break
         root = QtWidgets.QFileDialog.getExistingDirectory(
             self, translate('ImporterTab', "Select root folder"), directory)
         if not root:
             self._fail()
             return
-        if root in folders:
-            folders.remove(root)
-        folders.insert(0, root)
-        if len(folders) > 5:
-            del folders[-1]
-        self.config_store.set('importer', 'folders', repr(folders))
-        self.refresh()
-        idx = self.source_selector.count() - (1 + len(folders))
+        section = 'importer folder ' + root
+        self.config_store.set(
+            section, 'last_transfer', datetime.min.isoformat(' '))
+        self.source_selector.addItem(
+            translate('ImporterTab', 'folder: {0}').format(root),
+            (FolderSource(root), section))
+        idx = self.source_selector.count() - 1
         self.source_selector.setCurrentIndex(idx)
+        self.refresh()
 
     @QtSlot()
     @catch_all
@@ -432,7 +463,14 @@ class TabWidget(QtWidgets.QWidget):
             self.source_selector.addItem(
                 translate('ImporterTab', 'camera: {0}').format(model),
                 (CameraSource(model, port_name), 'importer ' + model))
-        for root in eval(self.config_store.get('importer', 'folders', '[]')):
+        roots = []
+        for section in self.config_store.config.sections():
+            if not section.startswith('importer folder '):
+                continue
+            roots.append((section[16:], self.config_store.get(
+                section, 'last_transfer', datetime.min.isoformat(' '))))
+        roots.sort(key=lambda x: x[1], reverse=True)
+        for root, last_transfer in roots:
             if os.path.isdir(root):
                 self.source_selector.addItem(
                     translate('ImporterTab', 'folder: {0}').format(root),
@@ -450,6 +488,7 @@ class TabWidget(QtWidgets.QWidget):
         self.source_selector.blockSignals(was_blocked)
         if new_idx < 0:
             self.source_selector.setCurrentIndex(0)
+            self.new_source(0)
 
     def do_not_close(self):
         if not self.file_copier:
@@ -457,20 +496,19 @@ class TabWidget(QtWidgets.QWidget):
         dialog = QtWidgets.QMessageBox(parent=self)
         dialog.setWindowTitle(translate(
             'ImporterTab', 'Photini: import in progress'))
-        dialog.setText(translate(
-            'ImporterTab', '<h3>Importing photos has not finished.</h3>'))
+        dialog.setText('<h3>{}</h3>'.format(translate(
+            'ImporterTab', 'Importing photos has not finished.')))
         dialog.setInformativeText(translate(
             'ImporterTab', 'Closing now will terminate the import.'))
         dialog.setIcon(QtWidgets.QMessageBox.Warning)
         dialog.setStandardButtons(
             QtWidgets.QMessageBox.Close | QtWidgets.QMessageBox.Cancel)
         dialog.setDefaultButton(QtWidgets.QMessageBox.Cancel)
-        result = dialog.exec_()
+        result = execute(dialog)
         if result == QtWidgets.QMessageBox.Close:
             self.stop_copy()
         return result == QtWidgets.QMessageBox.Cancel
 
-    @QtSlot(list)
     def new_selection(self, selection):
         pass
 
@@ -498,7 +536,7 @@ class TabWidget(QtWidgets.QWidget):
     @QtSlot()
     @catch_all
     def sort_file_list(self):
-        if eval(self.config_store.get('controls', 'sort_date', 'False')):
+        if self.config_store.get('controls', 'sort_date', False):
             self.file_list.sort(key=lambda x: self.file_data[x]['timestamp'])
         else:
             self.file_list.sort()
@@ -539,8 +577,8 @@ class TabWidget(QtWidgets.QWidget):
     @catch_all
     def selection_changed(self):
         count = len(self.file_list_widget.selectedItems())
-        self.selected_count.setText(
-            translate('ImporterTab', '%n file(s)\nselected', '', count))
+        self.selected_count.setText(wrap_text(self.selected_count, translate(
+            'ImporterTab', '%n file(s) selected', '', count), 2))
         if not self.file_copier:
             self.move_button.setEnabled(count > 0)
             self.copy_button.setEnabled(count > 0)
