@@ -1,6 +1,6 @@
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2018-20  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2018-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
 ##  along with this program.  If not, see
 ##  <http://www.gnu.org/licenses/>.
 
-from __future__ import unicode_literals
-
 import locale
 import logging
 
@@ -25,8 +23,8 @@ import requests
 
 from photini.configstore import key_store
 from photini.photinimap import GeocoderBase, PhotiniMap
-from photini.pyqt import (
-    Busy, catch_all, CompactButton, QtCore, QtGui, QtSlot, QtWidgets)
+from photini.pyqt import Busy, catch_all, QtCore, QtGui, QtSlot, QtWidgets
+from photini.widgets import CompactButton
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
@@ -34,8 +32,11 @@ translate = QtCore.QCoreApplication.translate
 
 class MapboxGeocoder(GeocoderBase):
     api_key = key_store.get('mapboxmap', 'api_key')
+    cache_size = 0
 
-    def do_geocode(self, query, params={}):
+    def query(self, params):
+        query = params['query']
+        del params['query']
         params['access_token'] = self.api_key
         params['autocomplete '] = 'false'
         lang, encoding = locale.getdefaultlocale()
@@ -60,7 +61,8 @@ class MapboxGeocoder(GeocoderBase):
 
     def search(self, search_string, bounds=None):
         params = {
-            'limit': 10,
+            'query': search_string,
+            'limit': '10',
             }
         if bounds:
             north, east, south, west = bounds
@@ -72,9 +74,9 @@ class MapboxGeocoder(GeocoderBase):
             if margin > 0.0:
                 north = min(north + margin,  90.0)
                 south = max(south - margin, -90.0)
-            params['bbox'] = '{!r},{!r},{!r},{!r}'.format(
+            params['bbox'] = '{:.4f},{:.4f},{:.4f},{:.4f}'.format(
                 west, south, east, north)
-        for feature in self.do_geocode(search_string, params=params):
+        for feature in self.query(params):
             if 'place_name' not in feature:
                 continue
             if 'bbox' in feature:
@@ -113,7 +115,5 @@ class TabWidget(PhotiniMap):
     </script>
     <script type="text/javascript">
       L.mapbox.accessToken = "{key}";
-    </script>
-    <script type="text/javascript" src="../openstreetmap/common.js">
     </script>'''.format(key=self.api_key,
-                        url='https://api.mapbox.com/mapbox.js/v3.2.1')
+                        url='https://api.mapbox.com/mapbox.js/v3.3.1')
