@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-20  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -17,86 +16,77 @@
 ##  along with this program.  If not, see
 ##  <http://www.gnu.org/licenses/>.
 
-from __future__ import unicode_literals
-
+from collections import defaultdict
 import logging
 import re
 import sys
 
-from photini.gi import Gspell, GSListPtr_to_list
+try:
+    import enchant
+except ImportError as ex:
+    enchant = None
+    print(str(ex))
+
 from photini.pyqt import catch_all, QtCore, QtSignal, QtSlot, QtWidgets
 
-spelling_version = None
-
-if not Gspell:
-    # avoid "dll Hell" on Windows by getting PyEnchant to use GObject's
-    # copy of libenchant and associated libraries
-    if sys.platform == 'win32':
-        # disable PyEnchant's forced use of its bundled DLLs
-        sys.platform = 'win32x'
-    try:
-        import enchant
-        spelling_version = 'enchant ' + enchant.__version__
-    except ImportError:
-        enchant = None
-    if sys.platform == 'win32x':
-        # reset sys.platform
-        sys.platform = 'win32'
-
 logger = logging.getLogger(__name__)
 
+if enchant:
+    spelling_version = 'PyEnchant ' + enchant.__version__
+else:
+    spelling_version = None
+
+
 class SpellCheck(QtCore.QObject):
     new_dict = QtSignal()
 
     def __init__(self, *arg, **kw):
         super(SpellCheck, self).__init__(*arg, **kw)
         self.config_store = QtWidgets.QApplication.instance().config_store
-        self.enable(eval(self.config_store.get('spelling', 'enabled', 'True')))
+        self.enable(self.config_store.get('spelling', 'enabled', True))
         self.set_language(self.config_store.get('spelling', 'language'))
 
     @staticmethod
     def available_languages():
-        result = []
-        if Gspell:
-            for lang in Gspell.Language.get_available():
-                result.append((lang.get_name(), lang.get_code()))
-        elif enchant:
-            languages = enchant.list_languages()
-            languages.sort()
-            for lang in languages:
-                result.append((lang, lang))
+        result = defaultdict(list)
+        if enchant:
+            for code in enchant.list_languages():
+                locale = QtCore.QLocale(code)
+                language = locale.languageToString(locale.language())
+                if '_' in code and '_ANY' not in code:
+                    country = locale.countryToString(locale.country())
+                else:
+                    country = ''
+                result[language].append((country, code))
         else:
             return None
-        return result
+        for value in result.values():
+            value.sort()
+        return dict(result) or None
 
     def current_language(self):
         if not self.dict:
             return ''
-        if Gspell:
-            language = self.dict.get_language()
-            if language:
-                return language.get_code()
-        elif enchant:
+        if enchant:
             return self.dict.tag
         return ''
 
     @QtSlot(bool)
     @catch_all
     def enable(self, enabled):
-        self.enabled = enabled and bool(Gspell or enchant)
-        self.config_store.set('spelling', 'enabled', str(self.enabled))
+        self.config_store.set('spelling', 'enabled', enabled)
+        self.enabled = enabled and bool(enchant)
         self.new_dict.emit()
 
     def set_language(self, code):
         if code:
             logger.debug('Setting dictionary %s', code)
         self.dict = None
-        if Gspell:
-            if code:
-                self.dict = Gspell.Checker.new(Gspell.Language.lookup(code))
-        elif enchant:
+        if enchant:
             if code and enchant.dict_exists(code):
                 self.dict = enchant.Dict(code)
+        else:
+            return
         if code and not self.dict:
             logger.warning('Failed to set dictionary %s', code)
         self.config_store.set('spelling', 'language', self.current_language())
@@ -111,8 +101,8 @@ class SpellCheck(QtCore.QObject):
     def check(self, word):
         if not (word and self.enabled and self.dict):
             return True
-        if Gspell:
-            return self.dict.check_word(word, -1)
+        if word.isnumeric():
+            return True
         if enchant:
             return self.dict.check(word)
         return True
@@ -120,8 +110,6 @@ class SpellCheck(QtCore.QObject):
     def suggest(self, word):
         if self.check(word):
             return []
-        if Gspell:
-            return GSListPtr_to_list(self.dict.get_suggestions(word, -1))
         if enchant:
             return self.dict.suggest(word)
         return []
