@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##  Photini - a simple photo metadata editor.
 ##  http://github.com/jim-easterbrook/Photini
-##  Copyright (C) 2012-21  Jim Easterbrook  jim@jim-easterbrook.me.uk
+##  Copyright (C) 2012-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
 ##
 ##  This program is free software: you can redistribute it and/or
 ##  modify it under the terms of the GNU General Public License as
@@ -17,218 +17,152 @@
 ##  along with this program.  If not, see
 ##  <http://www.gnu.org/licenses/>.
 
-from __future__ import unicode_literals
-
 from collections import defaultdict
 from datetime import datetime, timedelta
 import logging
-import math
 import re
 
-import six
-
-from photini.metadata import CameraModel, LensModel, LensSpec
-from photini.pyqt import (
-    catch_all, ComboBox, multiple, multiple_values, Qt, QtCore, QtGui,
-    QtSignal, QtSlot, QtWidgets, scale_font, set_symbol_font, Slider,
-    SquareButton, using_pyside2, width_for_text)
+from photini.pyqt import *
+from photini.pyqt import set_symbol_font, using_pyside
+from photini.types import MD_CameraModel, MD_LensModel
+from photini.widgets import DropDownSelector, Slider
 
 logger = logging.getLogger(__name__)
 translate = QtCore.QCoreApplication.translate
 
 
-class DropdownEdit(ComboBox):
-    extend_list = QtSignal()
-    new_value = QtSignal(object)
+class DropdownEdit(DropDownSelector):
+    def __init__(self, key, image_list, *arg, **kw):
+        super(DropdownEdit, self).__init__(
+            key, *arg, extendable=True, ordered=True, **kw)
+        self._image_list = image_list
+        self.config_store = QtWidgets.QApplication.instance().config_store
+        self.setFocusPolicy(Qt.NoFocus)
 
-    def __init__(self, extendable=False, **kw):
-        super(DropdownEdit, self).__init__(**kw)
-        if extendable:
-            self.setContextMenuPolicy(Qt.CustomContextMenu)
-            self.customContextMenuRequested.connect(self.remove_from_list)
-            self.addItem(
-                translate('TechnicalTab', '<new>'), self.extend_list.emit)
-        self.addItem('', None)
-        self.first_value_idx = self.count()
-        self.addItem(multiple_values(), '<multiple>')
-        self.setItemData(
-            self.count() - 1, self.itemData(self.count() - 1), Qt.UserRole - 1)
-        self.currentIndexChanged.connect(self.current_index_changed)
-
-    @QtSlot(QtCore.QPoint)
+    @QtSlot(QtGui.QContextMenuEvent)
     @catch_all
-    def remove_from_list(self, pos):
-        current_value = self.itemData(self.currentIndex())
+    def contextMenuEvent(self, event):
         menu = QtWidgets.QMenu()
-        for name, value in self.get_items():
-            if value == current_value:
+        for n in range(self._last_idx()):
+            if n == self.currentIndex():
                 continue
-            action = QtWidgets.QAction(
+            name = self.itemText(n)
+            if not name:
+                continue
+            action = QtGui2.QAction(
                 translate('TechnicalTab', 'Remove "{}"').format(name),
                 parent=self)
-            action.setData(value)
+            action.setData(self.itemData(n))
             menu.addAction(action)
         if menu.isEmpty():
             return
-        action = menu.exec_(self.mapToGlobal(pos))
+        action = execute(menu, event.globalPos())
         if not action:
             return
         self.remove_item(action.data())
 
-    @QtSlot(int)
-    @catch_all
-    def current_index_changed(self, idx):
-        value = self.itemData(idx)
-        if callable(value):
-            (value)()
-        else:
-            self.new_value.emit(value)
-
-    def add_item(self, text, value, ordered=True):
-        blocked = self.blockSignals(True)
-        position = self.count() - 1
-        if ordered:
-            for n in range(self.first_value_idx, self.count() - 1):
-                if self.itemText(n).lower() > text.lower():
-                    position = n
-                    break
-        self.insertItem(position, text, value)
-        self.set_dropdown_width()
-        self.blockSignals(blocked)
-
-    def remove_item(self, value):
-        blocked = self.blockSignals(True)
-        self.removeItem(self.find_data(value))
-        self.set_dropdown_width()
-        self.blockSignals(blocked)
-
-    def known_value(self, value):
-        if not value:
-            return True
-        return self.find_data(value) >= 0
-
-    def set_value(self, value):
-        blocked = self.blockSignals(True)
-        self.setCurrentIndex(self.find_data(value))
-        self.blockSignals(blocked)
-
-    def find_data(self, value):
-        # Qt's findData only works for simple types
-        for n in range(self.count()):
-            if self.itemData(n) == value:
-                return n
-        return -1
-
-    def get_items(self):
-        for n in range(self.first_value_idx, self.count() - 1):
-            yield self.itemText(n), self.itemData(n)
-
-    def set_multiple(self):
-        blocked = self.blockSignals(True)
-        self.setCurrentIndex(self.count() - 1)
-        self.blockSignals(blocked)
-
 
 class CameraList(DropdownEdit):
-    def __init__(self, **kw):
-        super(CameraList, self).__init__(**kw)
-        self.config_store = QtWidgets.QApplication.instance().config_store
+    def __init__(self, *args, **kwds):
+        super(CameraList, self).__init__(*args, **kwds)
+        values = [('', None)]
         # read cameras from config, updating if neccessary
-        camera_names = []
+        sections = []
         for section in self.config_store.config.sections():
-            if six.PY2:
-                section = section.decode('utf-8')
             if not section.startswith('camera '):
                 continue
-            camera_names.append(section[7:])
-        for camera_name in camera_names:
-            section = 'camera ' + camera_name
             camera = {}
             for key in 'make', 'model', 'serial_no':
                 camera[key] = self.config_store.get(section, key)
-            camera = CameraModel(camera)
+            camera = MD_CameraModel(camera)
             name = camera.get_name()
-            if name != camera_name:
+            if name != section[7:]:
                 self.config_store.remove_section(section)
-            self.add_item(camera)
+            values.append((name, camera))
+        self.set_values(values)
 
-    def add_item(self, camera):
+    def define_new_value(self):
+        dialog = NewCameraDialog(
+            self._image_list.get_selected_images(), parent=self)
+        if execute(dialog) != QtWidgets.QDialog.Accepted:
+            return None, None
+        camera = MD_CameraModel(dialog.get_value())
+        if not camera:
+            return None, None
         name = camera.get_name()
         section = 'camera ' + name
         for key, value in camera.items():
             if value:
                 self.config_store.set(section, key, value)
-        super(CameraList, self).add_item(name, camera, ordered=True)
+        return name, camera
 
-    def remove_item(self, camera):
+    def remove_item(self, camera, **kwds):
         self.config_store.remove_section('camera ' + camera.get_name())
-        super(CameraList, self).remove_item(camera)
+        return super(CameraList, self).remove_item(camera, **kwds)
 
-    def set_value(self, camera):
-        if not self.known_value(camera):
-            self.add_item(camera)
-        super(CameraList, self).set_value(camera)
+    def data_to_text(self, camera):
+        return camera.get_name()
 
 
 class LensList(DropdownEdit):
-    def __init__(self, **kw):
-        super(LensList, self).__init__(**kw)
-        self.config_store = QtWidgets.QApplication.instance().config_store
+    def __init__(self, *args, **kwds):
+        super(LensList, self).__init__(*args, **kwds)
+        values = [('', None)]
         # read lenses from config, updating if neccessary
         self.config_store.delete('technical', 'lenses')
-        lens_names = []
         for section in self.config_store.config.sections():
-            if six.PY2:
-                section = section.decode('utf-8')
             if not section.startswith('lens '):
                 continue
-            lens_names.append(section[5:])
-        for lens_name in lens_names:
-            section = 'lens ' + lens_name
             lens_model = {}
             for old_key, new_key in (('lens_make', 'make'),
                                      ('lens_model', 'model'),
-                                     ('lens_serial', 'serial_no')):
+                                     ('lens_serial', 'serial_no'),
+                                     ('lens_spec', 'spec')):
                 lens_model[new_key] = self.config_store.get(section, new_key)
                 if not lens_model[new_key]:
                     lens_model[new_key] = self.config_store.get(section, old_key)
                 self.config_store.delete(section, old_key)
-            lens_model = LensModel(lens_model)
-            lens_spec = LensSpec(self.config_store.get(section, 'lens_spec'))
-            if lens_model.get_name() != lens_name:
+            lens_model = MD_LensModel(lens_model)
+            name = lens_model.get_name()
+            if name != section[5:]:
                 self.config_store.remove_section(section)
-            self.add_item((lens_model, lens_spec))
+            values.append((name, lens_model))
+        self.set_values(values)
 
-    def add_item(self, value):
-        lens_model, lens_spec = value
+    def define_new_value(self):
+        dialog = NewLensDialog(
+            self._image_list.get_selected_images(), parent=self)
+        if execute(dialog) != QtWidgets.QDialog.Accepted:
+            return None, None
+        lens_model = dialog.get_value()
+        if not lens_model:
+            return None, None
         name = lens_model.get_name()
         section = 'lens ' + name
-        for k, v in lens_model.items():
-            if v:
-                self.config_store.set(section, k, v)
-        if lens_spec:
-            self.config_store.set(section, 'lens_spec', str(lens_spec))
-        super(LensList, self).add_item(name, value, ordered=True)
+        for key, value in lens_model.items():
+            if value:
+                self.config_store.set(section, key, str(value))
+        return name, lens_model
 
-    def remove_item(self, value):
-        lens_model, lens_spec = value
+    def remove_item(self, lens_model, **kwds):
         self.config_store.remove_section('lens ' + lens_model.get_name())
-        super(LensList, self).remove_item(value)
+        return super(LensList, self).remove_item(lens_model, **kwds)
 
-    def set_value(self, value):
-        if not self.known_value(value):
-            self.add_item(value)
-        super(LensList, self).set_value(value)
+    def data_to_text(self, lens_model):
+        return lens_model.get_name()
 
 
 class AugmentSpinBox(object):
     new_value = QtSignal(object)
 
-    def init_augment(self):
+    def __init__(self):
+        super(AugmentSpinBox, self).__init__()
+        if self.isRightToLeft():
+            self.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
         self.set_value(None)
         self.editingFinished.connect(self.editing_finished)
 
-    class ContextAction(QtWidgets.QAction):
+    class ContextAction(QtGui2.QAction):
         def __init__(self, value, *arg, **kw):
             super(AugmentSpinBox.ContextAction, self).__init__(*arg, **kw)
             self.setData(value)
@@ -259,6 +193,13 @@ class AugmentSpinBox(object):
             self.set_value(self.default_value)
             self.selectAll()
 
+    def fix_up(self):
+        if self.cleanText():
+            return False
+        # user has deleted the value
+        self.set_value(None)
+        return True
+
     @QtSlot()
     @catch_all
     def editing_finished(self):
@@ -297,7 +238,7 @@ class IntSpinBox(QtWidgets.QSpinBox, AugmentSpinBox):
         self.default_value = 0
         self.multiple = multiple_values()
         super(IntSpinBox, self).__init__(*arg, **kw)
-        self.init_augment()
+        AugmentSpinBox.__init__(self)
         self.setSingleStep(1)
         lim = (2 ** 31) - 1
         self.setRange(-lim, lim)
@@ -320,11 +261,7 @@ class IntSpinBox(QtWidgets.QSpinBox, AugmentSpinBox):
 
     @catch_all
     def fixup(self, text):
-        if not self.cleanText():
-            # user has deleted the value
-            self.set_value(None)
-            return ''
-        return super(IntSpinBox, self).fixup(text)
+        return self.fix_up() or super(IntSpinBox, self).fixup(text)
 
     def set_faint(self, faint):
         if faint:
@@ -338,7 +275,7 @@ class DoubleSpinBox(QtWidgets.QDoubleSpinBox, AugmentSpinBox):
         self.default_value = 0
         self.multiple = multiple_values()
         super(DoubleSpinBox, self).__init__(*arg, **kw)
-        self.init_augment()
+        AugmentSpinBox.__init__(self)
         self.setSingleStep(0.1)
         self.setDecimals(4)
         lim = (2 ** 31) - 1
@@ -362,16 +299,12 @@ class DoubleSpinBox(QtWidgets.QDoubleSpinBox, AugmentSpinBox):
 
     @catch_all
     def fixup(self, text):
-        if not self.cleanText():
-            # user has deleted the value
-            self.set_value(None)
-            return ''
-        return super(DoubleSpinBox, self).fixup(text)
+        return self.fix_up() or super(DoubleSpinBox, self).fixup(text)
 
     @catch_all
     def textFromValue(self, value):
         # don't use QDoubleSpinBox's fixed number of decimals
-        return str(round(value, self.decimals()))
+        return str(round(float(value), self.decimals()))
 
 
 class CalendarWidget(QtWidgets.QCalendarWidget):
@@ -393,7 +326,7 @@ class DateTimeEdit(QtWidgets.QDateTimeEdit, AugmentSpinBox):
         self.textFromValue = self.textFromDateTime
         self.value = self.dateTime
         super(DateTimeEdit, self).__init__(*arg, **kw)
-        self.init_augment()
+        AugmentSpinBox.__init__(self)
         self.setCalendarPopup(True)
         self.setCalendarWidget(CalendarWidget())
         self.precision = 1
@@ -454,7 +387,7 @@ class TimeZoneWidget(QtWidgets.QSpinBox, AugmentSpinBox):
         self.default_value = 0
         self.multiple = multiple()
         super(TimeZoneWidget, self).__init__(*arg, **kw)
-        self.init_augment()
+        AugmentSpinBox.__init__(self)
         self.setRange(-14 * 60, 15 * 60)
         self.setSingleStep(15)
         self.setWrapping(True)
@@ -476,11 +409,7 @@ class TimeZoneWidget(QtWidgets.QSpinBox, AugmentSpinBox):
 
     @catch_all
     def fixup(self, text):
-        if not self.cleanText():
-            # user has deleted the value
-            self.set_value(None)
-            return ''
-        return super(TimeZoneWidget, self).fixup(text)
+        return self.fix_up() or super(TimeZoneWidget, self).fixup(text)
 
     @catch_all
     def sizeHint(self):
@@ -548,7 +477,7 @@ class PrecisionSlider(Slider):
 
 
 class DateAndTimeWidget(QtWidgets.QGridLayout):
-    new_value = QtSignal(six.text_type, dict)
+    new_value = QtSignal(str, dict)
 
     def __init__(self, name, *arg, **kw):
         super(DateAndTimeWidget, self).__init__(*arg, **kw)
@@ -564,7 +493,7 @@ class DateAndTimeWidget(QtWidgets.QGridLayout):
         self.addWidget(self.members['tz_offset'], 0, 2)
         # precision
         self.addWidget(
-            QtWidgets.QLabel(translate('TechnicalTab', 'Precision:')), 1, 0)
+            QtWidgets.QLabel(translate('TechnicalTab', 'Precision')), 1, 0)
         self.members['precision'] = PrecisionSlider(Qt.Horizontal)
         self.members['precision'].setRange(1, 6)
         self.members['precision'].setValue(6)
@@ -588,7 +517,7 @@ class DateAndTimeWidget(QtWidgets.QGridLayout):
                 continue
             new_value[key] = self.members[key].get_value()
             if key == 'datetime' and new_value[key]:
-                if using_pyside2:
+                if using_pyside:
                     new_value[key] = new_value[key].toPython()
                 else:
                     new_value[key] = new_value[key].toPyDateTime()
@@ -621,22 +550,26 @@ class OffsetWidget(QtWidgets.QWidget):
         self.layout().addWidget(self.time_zone)
         self.layout().addSpacing(spacing)
         # add offset button
-        add_button = SquareButton(six.unichr(0x002b))
+        add_button = QtWidgets.QPushButton(chr(0x002b))
         add_button.setStyleSheet('QPushButton {padding: 0px}')
         set_symbol_font(add_button)
         scale_font(add_button, 170)
+        add_button.setFixedWidth(self.offset.sizeHint().height())
+        add_button.setFixedHeight(self.offset.sizeHint().height())
         add_button.clicked.connect(self.add)
         self.layout().addWidget(add_button)
         # subtract offset button
-        sub_button = SquareButton(six.unichr(0x2212))
+        sub_button = QtWidgets.QPushButton(chr(0x2212))
         sub_button.setStyleSheet('QPushButton {padding: 0px}')
         set_symbol_font(sub_button)
         scale_font(sub_button, 170)
+        sub_button.setFixedWidth(self.offset.sizeHint().height())
+        sub_button.setFixedHeight(self.offset.sizeHint().height())
         sub_button.clicked.connect(self.sub)
         self.layout().addWidget(sub_button)
         self.layout().addStretch(1)
         # restore stored values
-        value = eval(self.config_store.get('technical', 'offset', 'None'))
+        value = self.config_store.get('technical', 'offset')
         if value:
             self.offset.setTime(QtCore.QTime(*value[0:3]))
             self.time_zone.set_value(value[3])
@@ -661,7 +594,7 @@ class OffsetWidget(QtWidgets.QWidget):
         value = self.offset.time()
         value = (value.hour(), value.minute(), value.second(),
                  self.time_zone.get_value())
-        self.config_store.set('technical', 'offset', str(value))
+        self.config_store.set('technical', 'offset', value)
 
     @QtSlot()
     @catch_all
@@ -694,9 +627,7 @@ class NewItemDialog(QtWidgets.QDialog):
         scroll_area.setWidgetResizable(True)
         self.layout().addWidget(scroll_area)
         self.panel = QtWidgets.QWidget()
-        self.panel.setLayout(QtWidgets.QFormLayout())
-        self.panel.layout().setFieldGrowthPolicy(
-            QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
+        self.panel.setLayout(FormLayout())
         # ok & cancel buttons
         button_box = QtWidgets.QDialogButtonBox(
             QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)
@@ -748,12 +679,16 @@ class NewLensDialog(NewItemDialog):
         # fill in any values we can from existing metadata
         for image in images:
             model = image.metadata.lens_model
+            if not model:
+                continue
             for key in self.model_widgets:
-                if model and model[key]:
+                if model[key]:
                     self.model_widgets[key].setText(model[key])
-            spec = image.metadata.lens_spec
+            spec = model['spec']
+            if not spec:
+                continue
             for key in self.lens_spec:
-                if spec and spec[key]:
+                if spec[key]:
                     self.lens_spec[key].set_value(spec[key])
 
     def extend_data(self):
@@ -778,17 +713,16 @@ class NewLensDialog(NewItemDialog):
 
     def get_value(self):
         lens_model = super(NewLensDialog, self).get_value()
-        lens_model = LensModel(lens_model) or None
         min_fl = self.lens_spec['min_fl'].get_value() or 0
         max_fl = self.lens_spec['max_fl'].get_value() or min_fl
         min_fl_fn = self.lens_spec['min_fl_fn'].get_value() or 0
         max_fl_fn = self.lens_spec['max_fl_fn'].get_value() or min_fl_fn
-        lens_spec = LensSpec((min_fl, max_fl, min_fl_fn, max_fl_fn)) or None
-        return lens_model, lens_spec
+        lens_model['spec'] = (min_fl, max_fl, min_fl_fn, max_fl_fn)
+        return MD_LensModel(lens_model) or None
 
 
 class DateLink(QtWidgets.QCheckBox):
-    new_link = QtSignal(six.text_type)
+    new_link = QtSignal(str)
 
     def __init__(self, name, *arg, **kw):
         super(DateLink, self).__init__(*arg, **kw)
@@ -817,7 +751,7 @@ class TabWidget(QtWidgets.QWidget):
         # date and time
         date_group = QtWidgets.QGroupBox(
             translate('TechnicalTab', 'Date and time'))
-        date_group.setLayout(QtWidgets.QFormLayout())
+        date_group.setLayout(FormLayout())
         # create date and link widgets
         for master in self._master_slave:
             self.date_widget[master] = DateAndTimeWidget(master)
@@ -847,44 +781,44 @@ class TabWidget(QtWidgets.QWidget):
         self.layout().addWidget(date_group)
         # other
         other_group = QtWidgets.QGroupBox(translate('TechnicalTab', 'Other'))
-        other_group.setLayout(QtWidgets.QFormLayout())
-        other_group.layout().setFieldGrowthPolicy(
-            QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
+        other_group.setLayout(FormLayout())
         # orientation
-        self.widgets['orientation'] = DropdownEdit()
-        self.widgets['orientation'].add_item(
-            translate('TechnicalTab', 'normal'), 1, ordered=False)
-        self.widgets['orientation'].add_item(
-            translate('TechnicalTab', 'rotate -90'), 6, ordered=False)
-        self.widgets['orientation'].add_item(
-            translate('TechnicalTab', 'rotate +90'), 8, ordered=False)
-        self.widgets['orientation'].add_item(
-            translate('TechnicalTab', 'rotate 180'), 3, ordered=False)
-        self.widgets['orientation'].add_item(
-            translate('TechnicalTab', 'reflect left-right'), 2, ordered=False)
-        self.widgets['orientation'].add_item(
-            translate('TechnicalTab', 'reflect top-bottom'), 4, ordered=False)
-        self.widgets['orientation'].add_item(
-            translate('TechnicalTab', 'reflect tr-bl'), 5, ordered=False)
-        self.widgets['orientation'].add_item(
-            translate('TechnicalTab', 'reflect tl-br'), 7, ordered=False)
+        self.widgets['orientation'] = DropDownSelector(
+            'orientation', values=(
+                ('', None),
+                (translate('TechnicalTab', 'normal',
+                           'orientation dropdown, no transformation'), 1),
+                (translate('TechnicalTab', 'rotate -90°',
+                           'orientation dropdown'), 6),
+                (translate('TechnicalTab', 'rotate +90°',
+                           'orientation dropdown'), 8),
+                (translate('TechnicalTab', 'rotate 180°',
+                           'orientation dropdown'), 3),
+                (translate('TechnicalTab', 'reflect left to right',
+                           'orientation dropdown, horizontal reflection'), 2),
+                (translate('TechnicalTab', 'reflect top to bottom',
+                           'orientation dropdown, vertical reflection'), 4),
+                (translate('TechnicalTab', 'reflect top right to bottom left',
+                           'orientation dropdown, diagonal reflection'), 5),
+                (translate('TechnicalTab', 'reflect top left to bottom right',
+                           'orientation dropdown, diagonal reflection'), 7)))
         self.widgets['orientation'].new_value.connect(self.new_orientation)
+        self.widgets['orientation'].setFocusPolicy(Qt.NoFocus)
         other_group.layout().addRow(translate(
             'TechnicalTab', 'Orientation'), self.widgets['orientation'])
         # camera model
-        self.widgets['camera_model'] = CameraList(extendable=True)
+        self.widgets['camera_model'] = CameraList(
+            'camera_model', self.image_list)
         self.widgets['camera_model'].setMinimumWidth(
             width_for_text(self.widgets['camera_model'], 'x' * 30))
         self.widgets['camera_model'].new_value.connect(self.new_camera_model)
-        self.widgets['camera_model'].extend_list.connect(self.add_camera_model)
         other_group.layout().addRow(translate(
             'TechnicalTab', 'Camera'), self.widgets['camera_model'])
         # lens model
-        self.widgets['lens_model'] = LensList(extendable=True)
+        self.widgets['lens_model'] = LensList('lens_model', self.image_list)
         self.widgets['lens_model'].setMinimumWidth(
             width_for_text(self.widgets['lens_model'], 'x' * 30))
         self.widgets['lens_model'].new_value.connect(self.new_lens_model)
-        self.widgets['lens_model'].extend_list.connect(self.add_lens_model)
         other_group.layout().addRow(translate(
             'TechnicalTab', 'Lens model'), self.widgets['lens_model'])
         # focal length
@@ -919,7 +853,7 @@ class TabWidget(QtWidgets.QWidget):
         }
 
     def refresh(self):
-        pass
+        self.new_selection(self.image_list.get_selected_images())
 
     def do_not_close(self):
         return False
@@ -940,7 +874,7 @@ class TabWidget(QtWidgets.QWidget):
         self._update_datetime()
         self._update_links()
 
-    @QtSlot(six.text_type)
+    @QtSlot(str)
     @catch_all
     def new_link(self, master):
         slave = self._master_slave[master]
@@ -953,29 +887,17 @@ class TabWidget(QtWidgets.QWidget):
         else:
             self.date_widget[slave].set_enabled(True)
 
-    @QtSlot(object)
+    @QtSlot(str, object)
     @catch_all
-    def new_orientation(self, value):
+    def new_orientation(self, key, value):
         for image in self.image_list.get_selected_images():
             image.metadata.orientation = value
             image.load_thumbnail()
         self._update_orientation()
 
-    @QtSlot()
+    @QtSlot(str, object)
     @catch_all
-    def add_camera_model(self):
-        dialog = NewCameraDialog(
-            self.image_list.get_selected_images(), parent=self)
-        if dialog.exec_() == QtWidgets.QDialog.Accepted:
-            value = dialog.get_value()
-            if value:
-                self.new_camera_model(value)
-                return
-        self._update_camera_model()
-
-    @QtSlot(object)
-    @catch_all
-    def new_camera_model(self, value):
+    def new_camera_model(self, key, value):
         delete_makernote = 'ask'
         for image in self.image_list.get_selected_images():
             if not image.metadata.camera_change_ok(value):
@@ -983,45 +905,27 @@ class TabWidget(QtWidgets.QWidget):
                     msg = QtWidgets.QMessageBox(parent=self)
                     msg.setWindowTitle(translate(
                         'TechnicalTab', 'Photini: maker name change'))
-                    msg.setText(translate(
-                        'TechnicalTab', '<h3>Changing maker name will'
-                        ' invalidate Exif makernote information.</h3>'))
+                    msg.setText('<h3>{}</h3>'.format(translate(
+                        'TechnicalTab', 'Changing maker name will'
+                        ' invalidate Exif makernote information.')))
                     msg.setInformativeText(translate(
                         'TechnicalTab',
                         'Do you want to delete the Exif makernote?'))
                     msg.setIcon(msg.Warning)
                     msg.setStandardButtons(msg.YesToAll | msg.NoToAll)
                     msg.setDefaultButton(msg.NoToAll)
-                    delete_makernote = msg.exec_() == msg.YesToAll
+                    delete_makernote = execute(msg) == msg.YesToAll
                 if delete_makernote:
                     image.metadata.set_delete_makernote()
             image.metadata.camera_model = value
         self._update_camera_model()
 
-    @QtSlot()
+    @QtSlot(str, object)
     @catch_all
-    def add_lens_model(self):
-        dialog = NewLensDialog(
-            self.image_list.get_selected_images(), parent=self)
-        if dialog.exec_() == QtWidgets.QDialog.Accepted:
-            value = dialog.get_value()
-            if value[0]:
-                self.new_lens_model(value)
-                return
-        self._update_lens_model()
-
-    @QtSlot(object)
-    @catch_all
-    def new_lens_model(self, value):
-        if value:
-            lens_model, lens_spec = value
-        else:
-            lens_model, lens_spec = None, None
+    def new_lens_model(self, key, value):
         for image in self.image_list.get_selected_images():
-            image.metadata.lens_model = lens_model
-            image.metadata.lens_spec = lens_spec
+            image.metadata.lens_model = value
         self._update_lens_model()
-        self._update_lens_spec()
 
     @QtSlot(object)
     @catch_all
@@ -1051,7 +955,7 @@ class TabWidget(QtWidgets.QWidget):
         self._update_focal_length()
         self._update_focal_length_35()
 
-    @QtSlot(six.text_type, dict)
+    @QtSlot(str, dict)
     @catch_all
     def new_date_value(self, key, new_value):
         for image in self.image_list.get_selected_images():
@@ -1115,7 +1019,7 @@ class TabWidget(QtWidgets.QWidget):
                 # multiple values
                 self.widgets['orientation'].set_multiple()
                 return
-        self.widgets['orientation'].set_value(value)
+        self.widgets['orientation'].set_value(value and int(value))
 
     def _update_camera_model(self):
         images = self.image_list.get_selected_images()
@@ -1133,29 +1037,18 @@ class TabWidget(QtWidgets.QWidget):
         if not images:
             return
         self.widgets['lens_model'].setToolTip('')
-        value = images[0].metadata.lens_model, images[0].metadata.lens_spec
+        value = images[0].metadata.lens_model
         for image in images[1:]:
-            if (image.metadata.lens_model, image.metadata.lens_spec) != value:
+            if image.metadata.lens_model != value:
                 # multiple values
                 self.widgets['lens_model'].set_multiple()
                 return
-        lens_model, lens_spec = value
-        if lens_spec and not lens_model:
-            lens_model = '{}'.format(float(lens_spec['min_fl']) or '')
-            if lens_spec['max_fl'] != lens_spec['min_fl']:
-                lens_model += '-{}'.format(float(lens_spec['max_fl']) or '')
-            lens_model += ' mm'
-            if lens_spec['min_fl_fn']:
-                lens_model += ' 1:{}'.format(float(lens_spec['min_fl_fn']) or '')
-            if lens_spec['max_fl_fn'] != lens_spec['min_fl_fn']:
-                lens_model += '-{}'.format(float(lens_spec['max_fl_fn']) or '')
-            lens_model = LensModel({'model': lens_model})
-            value = lens_model, lens_spec
-        if not lens_model:
-            value = None
         self.widgets['lens_model'].set_value(value)
-        if not lens_spec:
-            lens_spec = LensSpec(None)
+        if not value:
+            return
+        spec = value['spec']
+        if not spec:
+            return
         tool_tip = ('<table><tr><th></th><th width="70">{th_min}</th>'
                     '<th width="70">{th_max}</th></tr>'
                     '<tr><th align="right">{th_fl}</th>'
@@ -1169,20 +1062,8 @@ class TabWidget(QtWidgets.QWidget):
             th_max=translate('TechnicalTab', 'max'),
             th_fl=translate('TechnicalTab', 'Focal length'),
             th_ap=translate('TechnicalTab', 'Max aperture'),
-            **dict([(x, float(y) or '') for (x, y) in lens_spec.items()]))
+            **dict([(x, float(y) or '') for (x, y) in spec.items()]))
         self.widgets['lens_model'].setToolTip(tool_tip)
-
-    def _update_lens_spec(self):
-        images = self.image_list.get_selected_images()
-        if not images:
-            return
-        spec = images[0].metadata.lens_spec
-        if not spec:
-            return
-        for image in images[1:]:
-            if image.metadata.lens_spec != spec:
-                # multiple values
-                return
         make_changes = False
         for image in images:
             new_aperture = image.metadata.aperture or 0
@@ -1287,43 +1168,25 @@ class TabWidget(QtWidgets.QWidget):
         if not md.camera_model:
             return
         if not md.focal_length_35:
-            self.config_store.set(
-                'crop factor', md.camera_model.get_name(inc_serial=False),
-                'None')
+            self.config_store.delete(
+                'crop factor', md.camera_model.get_name(inc_serial=False))
         elif md.focal_length:
             crop_factor = float(md.focal_length_35) / md.focal_length
             self.config_store.set(
                 'crop factor', md.camera_model.get_name(inc_serial=False),
-                repr(crop_factor))
+                crop_factor)
 
     def get_crop_factor(self, md):
         if md.camera_model:
             crop_factor = self.config_store.get(
                 'crop factor', md.camera_model.get_name(inc_serial=False))
             if crop_factor:
-                return eval(crop_factor)
-        if not all((md.resolution_x, md.resolution_y,
-                    md.dimension_x, md.dimension_y)):
-            return None
-        if (md.resolution_x <= 0 or md.resolution_y <= 0 or
-                md.dimension_x <= 0 or md.dimension_y <= 0):
-            return None
-        # calculate from image size and resolution
-        w = md.dimension_x / md.resolution_x
-        h = md.dimension_y / md.resolution_y
-        d = math.sqrt((h ** 2) + (w ** 2))
-        if md.resolution_unit == 3:
-            # unit is cm
-            d *= 10.0
-        elif md.resolution_unit in (None, 1, 2):
-            # unit is (assumed to be) inches
-            d *= 25.4
-        # 35 mm film diagonal is 43.27 mm
-        crop_factor = round(43.27 / d, 4)
-        if md.camera_model:
+                return crop_factor
+        crop_factor = md.get_crop_factor()
+        if crop_factor and md.camera_model:
             self.config_store.set(
                 'crop factor', md.camera_model.get_name(inc_serial=False),
-                str(crop_factor))
+                crop_factor)
         return crop_factor
 
     def calc_35(self, md, value=None):
@@ -1333,8 +1196,6 @@ class TabWidget(QtWidgets.QWidget):
             return int((float(value) * crop_factor) + 0.5)
         return md.focal_length_35
 
-    @QtSlot(list)
-    @catch_all
     def new_selection(self, selection):
         if not selection:
             self.setEnabled(False)
